/* ============================================================================================================================================= /
 * PROJECT:         Orologio GLCD
 * FILE:            Main.c
 * LAST REV:        2.4
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F4550 su FREEDOM 2
 * CREATED:         01/12/2013
 * UPDATED:         13/05/2014
 *
 * DESCRIP:         Orologio digitale/sveglia/temperatura
 *
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 2.4              13/05/2014          - Gestita temperatura anche su Freedom II
 * 2.3              12/04/2014          - Inserito impostazione treshold all'avvio e salvataggio in EEPROM
 * 2.2              07/03/2014          - Attivazione buzzer da interfaccia utente
 * 2.1              20/02/2014          - inserito settaggio per attivare o disattivare il buzzer
 *                                      - SW1 e SW2 stesse funzioni quando non in set mode
 * 2.0              10/02/2014          - Porting da LCD 2x16 a GLCD con libreria GLCD
 *                                      - Gestione retroilluminazione in PWM con luminosit a 4 livelli (2 per il giorno e 2 per la notte)
 *                                      - implementato sensore di luminosit
 * 1.2              05/12/2013          - Lampeggio al primo avvio
 *                                      - aggiornamento temperatura ogni 10 secondi
 *                                      - bugfix
 * 1.1              04/12/2013          Prima release:
 *                                      - Impostazione/visualizzazione data ora e giorno settimana
 *                                      - Disposizione campi visualizzazione su LCD impostabili in Main.h
 *                                      - Allarme ora attivabile per giorni settimana multipli, con disattivazione globale
 *                                      - Retroilluminazione attivabile per tempo personalizzabile
 *                                      - Bozza visualizzazione temperatura
 * 1.0              01/12/2013          - inizio sviluppo
/* ============================================================================================================================================ */

/* INCLUDES =================================================================================================================================== */
#include <xc.h>
#include "Main.h"
#include "Prototipi.h"
#include "Delay.c"
#include "Manage_input.c"
#include "GLCD_KS0108B.c"
#include "EEPROM.C"

#include "I2C_init.c"
#include "RTCC_PCF8563.c"

/* GLOBAL VARIABLES =========================================================================================================================== */
bit  new_day;                                                                                       // flag ora a 00:00:00 per aggiornare data e giorno
bit  ten_sec;                                                                                       // flag 10 secondi
bit  alarm;                                                                                         // flag allarme impostato
char alarm_by_day;                                                                                  // bite per indicare in quali giorni va attivato l'allarme
                                                                                                    // ogni bit un giorno nell'ordine: xSVGMMLD
int light_treshold;
char duty_low;                                                                                      // variabili duty cicle
char duty_high;
bit  reset;                                                                                         // bit reset avvenuto
bit  alarm_int_occured;
bit  tone;
bit set_up_treshold;

/* ============================================================================================================================================= /
 * FUNCTION:       Interrupts
 *
 * Overview:
/* ============================================================================================================================================ */
void interrupt Inter (void) {
    if (INTCON3bits.INT2IF) {                                                                       // se arriva interrupt da INT2
        T1CONbits.TMR1ON = 1;                                                                       // avvia timer 1 (allarme)
        alarm_int_occured = 1;                                                                      // setto flag allarme scattato
        INTCON3bits.INT2IF = 0;                                                                     // resetta flag
    }

    if (INTCONbits.TMR0IF) {                                                                        // se l'interrupt proviene dal timer 0 (1 interrupt ogni 250ms)
        static char _timer0;
        TMR0 = 26476;                                                                               // resetto il preload
        _timer0++;                                                                                  // incremento il contatore su variabile

        if (_timer0 == BACK_ON * 4) {                                                               // se il contaotore arriva a al valore stabilito (BACK_ON * 4 perch ho un interrupt ogni 250ms)
            CCPR2L = duty_low;                                                                      // spengo retroilluminazione
            T0CONbits.TMR0ON = 0;                                                                   // spengo il timer
            _timer0 = 0;                                                                            // resetto contatore
        }
        INTCONbits.TMR0IF = 0;                                                                      // resetto il flag di interrupt da timer 0
    }

    if (PIR1bits.TMR1IF) {                                                                          // se l'interrupt proviene dal timer 1 (1 interrupt ogni 100ms)
        static char _timer1;
        TMR0 = 3038;                                                                                // resetto il preload
        _timer1++;                                                                                  // incremento il contatore su variabile globale (per reset)

        if (_timer1 == 5) {                                                                         // se il contaotore arriva a 5 (5*100ms = 500ms)
            if (CCPR2L == duty_high)                                                                        // alterno dity cilce 0/alto
                CCPR2L = 0;
            else
                CCPR2L = duty_high;

            if (tone)
                BUZZ = !BUZZ;                                                                       // allarme alternato se flag BUZZER_ON  settato

            _timer1 = 0;                                                                            // resetto contatore
        }
        PIR1bits.TMR1IF = 0;                                                                        // resetto il flag di interrupt da timer 0
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        Main
 *
 * Overview:        Ciclo principale: dopo i settaggi entra in ciclo continuo per aggiornare l'ora e la data (solo se ora  00:00:00) e temperatura
 *                  (ogni 10 secondi), entra in setup se premuto sw4, entra in setup allarme se premuto sw3, accende retroilluminazione se premuto
 *                  sw1, spegne allarme se premuto sw2.
/* ============================================================================================================================================ */

void main(void) {
    Define_IO();
    Module_settings();

    light_treshold = EEPROM_read_int (0x00);

    GLCD_init();
    I2C_init_master (400);

    if (Debounce_single(SW4)) {
        Set_treshold();
    }

    reset = 1;

    RTCC_write_day(0x01);                                                                           // setto data di partenza
    RTCC_write_month(0x01);
    RTCC_write_year(0x14);
    CCPR2L = DUTY_LOW_DAY;                                                                          // setto duty cicle di partenza

    Drow_basic_layout();                                                                            // disegno la cornice

    while(1) {

        Update_time();                                                                              // aggiorno ora su GLCD
        Check_light();                                                                              // controllo il sensore di luminosit

        if (new_day || reset) {                                                                     // se il flag new_day o reset  stato settato
            Update_day_w();                                                                         // aggiorno anche il giorno
            Update_date();                                                                          // e la data
            Alarm_check_day(0);                                                                     // controllo se nel nuovo giorno e' settato l'allarme
            new_day = 0;                                                                            // resetto flag new_day
        }

        if (Debounce_single(SW4)) {                                                                 // se premo SW4
            Set_mode();                                                                             // vado ad impostare data e ora
        }

        if (Debounce_single(SW3)) {                                                                 // se premo SW3
            Set_alarm_mode();                                                                       // vado ad impostare l'allarme
        }

        if (Debounce_single(SW1) || Debounce_single(SW2)) {                                         // se premo SW1 o SW2
            if (alarm_int_occured) {                                                                // se l'allarme  scattato
                RTCC_clear_alarm();                                                                 // resetto il flag di allarme avvenuto nell'RTCC
                T1CONbits.TMR1ON = 0;                                                               // fermo il timer1
                BUZZ = OFF;                                                                         // e spengo l'allarme
                CCPR2L = duty_low;                                                                  // forzo duty cicle basso
                alarm_int_occured = 0;                                                              // resetto flag allarme scattato
                Alarm_check_day(1);                                                                 // controllo se l'allarme va abilitato anche sul nuovo giorno.
            } else {                                                                                // altrimenti
                CCPR2L = duty_high;                                                                 // imposto duty cicle alto per aumentare la luminosit
                T0CONbits.TMR0ON = 1;                                                               // e attivo relativo timer per diminuire la luminosit dopo tot secondi
            }
        }

        if (ten_sec && !reset) {                                                                    // se flag 10 secondi = 1 ed  gia stata impostata l' ora
            Update_temp();                                                                          // aggiorno la temperatura
            ten_sec = 0;                                                                            // resetto flag
        }

        if (reset) {                                                                                // se flag di reset  settato (solo all'avvio dopo reset fino a prima impostazione)
            DelayMs(500);                                                                           // faccio lampeggiare le scritte
            GLCD_clear();                                                                           // cancellando il display
            DelayMs(500);
            Drow_basic_layout();
        }

    }
}

void Drow_basic_layout(void) {
    GLCD_clear();
    GLCD_drow_block (1, 128, 1, 64, ON);
    GLCD_drow_block (2, 127, 2, 63, ON);
}


/* ============================================================================================================================================= /
 * FUNCTION:        Update_time
 * Conditions:      definire in setup TIME_ROW e TIME_COL (posizione su LCD dell'ora)
 *
 * Overview:        Scrive su LCD l'ora e setta il flag new day se l'ora  00:00:00
/* ============================================================================================================================================ */

void Update_time(void) {
    static char _hours_p = 0;
    static char _minutes_p = 0;
    static char _new_day_checked;
    static char _ten_sec_checked;

    char _hours = RTCC_read_hours();                                                                // salvo l'ora in variabili locali
    char _minutes = RTCC_read_minutes();
    char _seconds = RTCC_read_seconds();

    if ((_hours != _hours_p) || (reset) || (new_day)) {                                             // se le ore attuali sono diverse dalle ore della lettura precedente o se arrivo da reset o da new day
        GLCD_fill_block(TIME_START_X, TIME_START_X +42 , TIME_START_Y -31 , TIME_START_Y, OFF);     // cancello la scritta precedente
        GLCD_write_PBCD(TIME_START_X, TIME_START_Y, _hours, FONT_20x31);                            // scrivo le ore sul display
        _hours_p = _hours;                                                                          // e imposto ore precedenti
    }

    if ((_minutes != _minutes_p) || (reset) || (new_day)) {                                         // se i minuti attuali sono diversi dai minuti della lettura precedente o se arrivo da reset o da new day
        GLCD_fill_block(TIME_START_X +56, TIME_START_X +98 , TIME_START_Y -31 , TIME_START_Y, OFF); // cancello la scritta precedente
        GLCD_write_PBCD(TIME_START_X +56 , TIME_START_Y, _minutes, FONT_20x31);                     // scrivo i minuti sul display
        _minutes_p = _minutes;                                                                      // e imposto minuti precedenti
    }

    GLCD_write_char(TIME_START_X + 39, TIME_START_Y,':', FONT_20x31);                               // scrivo i 2 punti

    if (_hours == 0 && _minutes == 0 && _seconds == 0 && _new_day_checked == 0) {                   // se l'ora  00:00:00 e _new_day non  ancora stato verificato
        new_day = 1;                                                                                // setto new_day
        _new_day_checked = 1;                                                                       // e new day checked
    } else if (_seconds != 0){                                                                      // se secondi sono diversi da 0
        _new_day_checked = 0;                                                                       // resetto _new_day_checked (serve per rilevare new day solo una volta)
    }
    if ((_seconds & 0b00001111) == 0 && _ten_sec_checked == 0) {                                    // ogni 10 secondi setto ten_sec e ten_sec_cheked
        ten_sec = 1;
        _ten_sec_checked = 1;                                                                       // resetto ten_sec_cheked quando i secondi sono diversi da 0
    } else if ((_seconds & 0b00001111) != 0 ){                                                      // (serve per rilevare 10 secondi solo una volta)
        _ten_sec_checked = 0;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        Update_date
 * Conditions:      definire in setup DATE_ROW e DATE_COL (posizione su LCD dell'ora)
 *
 * Overview:        Scrive su LCD la data
/* ============================================================================================================================================ */

void Update_date(void) {
    GLCD_fill_block(DATE_START_X, DATE_START_X +63 , DATE_START_Y -7, DATE_START_Y, OFF);           // cancello scritta precendente
    GLCD_write_PBCD(DATE_START_X, DATE_START_Y, RTCC_read_day(), FONT_7x7);                         // scrivo il giorno
    GLCD_write_char(DATE_START_X + 16, DATE_START_Y, '/', FONT_7x7);
    GLCD_write_PBCD(DATE_START_X + 25, DATE_START_Y, RTCC_read_month(), FONT_7x7);
    GLCD_write_char(DATE_START_X + 41, DATE_START_Y, '/', FONT_7x7);
    GLCD_write_PBCD(DATE_START_X + 49, DATE_START_Y, 0x20, FONT_7x7);
    GLCD_write_PBCD(DATE_START_X + 64, DATE_START_Y, RTCC_read_year(), FONT_7x7);
}

/* ============================================================================================================================================= /
 * FUNCTION:        Update_day_w
 * Conditions:      definire in setup DAY_ROW e DAY_COL (posizione su LCD dell'ora)
 *
 * Overview:        Scrive su LCD il giorno della settimana
/* ============================================================================================================================================ */

void Update_day_w(void) {
    char _day_w = RTCC_read_day_w();                                                                // leggo il giorno della settimana dall'RTCC
    GLCD_fill_block(DAY_START_X, DAY_START_X +71 , DAY_START_Y -7 , DAY_START_Y, OFF);              // cancello scritta precendente
    Write_day_w_GLCD(_day_w);                                                                       // scrivo il giorno sul display
}

void Write_day_w_GLCD(char _day_w) {                                                                // decodifica del nome del giorno
    switch(_day_w) {
        case 0: GLCD_write_string(DAY_START_X, DAY_START_Y, D0, FONT_7x7);
        break;
        case 1: GLCD_write_string(DAY_START_X, DAY_START_Y, D1, FONT_7x7);
        break;
        case 2: GLCD_write_string(DAY_START_X, DAY_START_Y, D2, FONT_7x7);
        break;
        case 3: GLCD_write_string(DAY_START_X, DAY_START_Y, D3, FONT_7x7);
        break;
        case 4: GLCD_write_string(DAY_START_X, DAY_START_Y, D4, FONT_7x7);
        break;
        case 5: GLCD_write_string(DAY_START_X, DAY_START_Y, D5, FONT_7x7);
        break;
        case 6: GLCD_write_string(DAY_START_X, DAY_START_Y, D6, FONT_7x7);
        break;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        Set_mode
 *
 * Overview:        Impostazione data/ora/giorno, ad ogni pressione di SW4 si passa al parametro successivo in ordine:
 *                  GG->MM->AA->hh->mm->ss->DW
/* ============================================================================================================================================ */

void Set_mode(void) {

    T0CONbits.TMR0ON = 0;                                                                           // forzo spegnimento timer0
    CCPR2L = duty_high;                                                                             // alzo la retroilluminazione
    new_day = 1;                                                                                    // forzo flag new_day per aggiornare data all'uscita della funzione
    reset = 0;                                                                                      // resetto flag di reset
    ten_sec = 1;                                                                                    // forzo flag 10 secondi per scrivere la temperatura all'uscita

    char _minutes = RTCC_read_minutes();                                                            // leggo ora e data e lisalvo in variabili locali
    char _hours = RTCC_read_hours();
    char _day = RTCC_read_day();
    char _month = RTCC_read_month();
    char _year = RTCC_read_year();
    char _day_w = RTCC_read_day_w();

    do {                                                                                            // IMPOSTAZIONE GIORNO
        if (Debounce_multi(SW2)) {                                                                  // se premo SW2
            _day = Inc_pbcd(_day, MIN_DAY, MAX_DAY);                                                // incremento la variabile temporanea dei giorni
        }

        if (Debounce_multi(SW1)) {                                                                  // se premo SW1
            _day = Dec_pbcd(_day, MIN_DAY, MAX_DAY);                                                // decremento la variabile temporanea dei giorni
        }
        DelayMs(100);
        GLCD_fill_block(DATE_START_X, DATE_START_X +15, DATE_START_Y -7, DATE_START_Y, OFF);        // cancello dato precedente
        DelayMs(100);
        GLCD_write_PBCD(DATE_START_X, DATE_START_Y, _day, FONT_7x7);                                // scrivo nuovo dato

    } while (!Debounce_single(SW4));                                                                // ripeto fin che non viene premuto SW4

    do {                                                                                            // IMPOSTAZIONE MESE
        if (Debounce_multi(SW2)) {                                                                  // se premo SW2
            _month = Inc_pbcd(_month, MIN_MONTH, MAX_MONTH);                                        // incremento la variabile temporanea del mese
        }

        if (Debounce_multi(SW1)) {                                                                  // se premo SW1
            _month = Dec_pbcd(_month, MIN_MONTH, MAX_MONTH);                                        // decremento la variabile temporanea del mese
        }
        DelayMs(100);
        GLCD_fill_block(DATE_START_X +25, DATE_START_X +39, DATE_START_Y -7, DATE_START_Y, OFF);
        DelayMs(100);
        GLCD_write_PBCD(DATE_START_X +25, DATE_START_Y, _month, FONT_7x7);

    } while (!Debounce_single(SW4));                                                                // ripeto fin che non viene premuto SW4

    do {                                                                                            // IMPOSTAZIONE ANNO
        if (Debounce_multi(SW2)) {
            _year = Inc_pbcd(_year, MIN_YEAR, MAX_YEAR);
        }

        if (Debounce_multi(SW1)) {
            _year = Dec_pbcd(_year, MIN_YEAR, MAX_YEAR);
        }
        DelayMs(100);
        GLCD_fill_block(DATE_START_X +49, DATE_START_X +78, DATE_START_Y -7, DATE_START_Y, OFF);
        DelayMs(100);
        GLCD_write_PBCD(DATE_START_X + 49, DATE_START_Y, 0x20, FONT_7x7);
        GLCD_write_PBCD(DATE_START_X +64, DATE_START_Y, _year, FONT_7x7);

    } while (!Debounce_single(SW4));

    do {                                                                                            // IMPOSTAZIONE ORE
        if (Debounce_multi(SW2)) {
            _hours = Inc_pbcd(_hours, MIN_HOU, MAX_HOU);
        }

        if (Debounce_multi(SW1)) {
            _hours = Dec_pbcd(_hours, MIN_HOU, MAX_HOU);
        }
        DelayMs(100);
        GLCD_fill_block(TIME_START_X, TIME_START_X +42, TIME_START_Y -31, TIME_START_Y, OFF);
        DelayMs(100);
        GLCD_write_PBCD(TIME_START_X, TIME_START_Y, _hours, FONT_20x31);

    } while (!Debounce_single(SW4));

    do {                                                                                            // IMPOSTAZIONE MINUTI
        if (Debounce_multi(SW2)) {
            _minutes = Inc_pbcd(_minutes, MIN_MIN, MAX_MIN);
        }

        if (Debounce_multi(SW1)) {
            _minutes = Dec_pbcd(_minutes, MIN_MIN, MAX_MIN);
        }
        DelayMs(100);
        GLCD_fill_block(TIME_START_X +56, TIME_START_X +98, TIME_START_Y -31, TIME_START_Y, OFF);
        DelayMs(100);
        GLCD_write_PBCD(TIME_START_X +56, TIME_START_Y, _minutes, FONT_20x31);

    } while (!Debounce_single(SW4));

    do {                                                                                            // IMPOSTAZIONE GIORNO DELLA SETTIMANA
        if (Debounce_multi(SW2)) {
        _day_w++;
            if (_day_w > MAX_DAY_WEEK) _day_w = 0;
        }

        if (Debounce_multi(SW1)) {
       _day_w--;
            if (_day_w > MAX_DAY_WEEK) _day_w = MAX_DAY_WEEK;
        }
        DelayMs(100);
        GLCD_fill_block(DAY_START_X, DAY_START_X +71, DAY_START_Y -7, DAY_START_Y, OFF);
        DelayMs(100);
        Write_day_w_GLCD(_day_w);

    } while (!Debounce_single(SW4));

    RTCC_write_seconds(00);                                                                         // salvo i valori impostati nell'RTCC
    RTCC_write_minutes(_minutes);
    RTCC_write_hours(_hours);
    RTCC_write_day(_day);
    RTCC_write_month(_month);
    RTCC_write_year(_year);
    RTCC_write_day_w(_day_w);

    T0CONbits.TMR0ON = 1;                                                                           // avvio timer0 per spegnere retroilluminazione dopo 5 secondi
}

/* ============================================================================================================================================= /
 * FUNCTION:        Set_alarm_mode
 *
 * Overview:        Impostazione ora/giorno della settimana, abilitazione dell'allarme, ad ogni pressione di SW4 si passa al parametro successivo
 *                  in ordine: hh->mm->D->L->M->M->G->V->S->ON/OFF
/* ============================================================================================================================================ */

void Set_alarm_mode(void) {

    T0CONbits.TMR0ON = 0;                                                                           // forzo spegnimento timer 0
    CCPR2L = duty_high;                                                                             // alzo la retroilluminazione

    new_day = 1;                                                                                    // forzo aggiornamento data e giorno
    ten_sec = 1;                                                                                    // forzo aggiornamento temperatura

    char _alarm_by_day_temp = alarm_by_day;                                                         // salvo il byte con i giorni attivi in una variabile temporanea
    char _minutes_al = RTCC_read_minutes_al();                                                      // salvo ora dell'allarme in variabili temporanee
    char _hours_al = RTCC_read_hours_al();

    Drow_basic_layout();

    GLCD_write_string(ALARM_WRITE_X, ALARM_WRITE_Y, "ALARM", FONT_4x5);                             // scrivo ALARM

    if (alarm)                                                                                      // scrivo ON OFF nella posizione impostata se allarme attivo
        GLCD_write_string(ALARM_ON_X, ALARM_ON_Y, "ON", FONT_4x5);
    else
        GLCD_write_string(ALARM_ON_X, ALARM_ON_Y, "OFF", FONT_4x5);

    GLCD_write_string(TONE_X, TONE_Y, "TONE", FONT_4x5);                                            // scrivo TONE

    if (tone)                                                                                       // scrivo ON OFF nella posizione impostata se allarme attivo
        GLCD_write_string(TONE_ON_X, TONE_ON_Y, "ON", FONT_4x5);
    else
        GLCD_write_string(TONE_ON_X, TONE_ON_Y, "OFF", FONT_4x5);

    GLCD_write_PBCD(ALARM_TIME_START_X, ALARM_TIME_START_Y, _hours_al, FONT_20x31);                 // scrivo l'ora attuale dell'allarme
    GLCD_write_PBCD(ALARM_TIME_START_X +56 , ALARM_TIME_START_Y, _minutes_al, FONT_20x31);
    GLCD_write_char(ALARM_TIME_START_X + 39, ALARM_TIME_START_Y,':', FONT_20x31);

    GLCD_write_string(ALARM_BY_DAY_START_X, ALARM_BY_DAY_START_Y, "D L M M G V S", FONT_7x7);       // scrivo la lista dei giorni nella posizione impostata

    for (char _i=0; _i<7; _i++ ){                                                                   // scrivo sotto la lista dei giorni una X sui giorni che hanno l'allarme abilitato
        if (_alarm_by_day_temp & 0b00000001)
            GLCD_write_char(ALARM_BY_DAY_START_X + (_i*4) + (_i*7), ALARM_BY_DAY_START_Y + 9,'X', FONT_7x7);
//        else
//            GLCD_fill_block(ALARM_BY_DAY_START_X + (_i*4) + (_i*7), ALARM_BY_DAY_START_X + (_i*4) + (_i*7) + 7, ALARM_BY_DAY_START_Y + 9,ALARM_BY_DAY_START_Y +9 -7,OFF);
        _alarm_by_day_temp = _alarm_by_day_temp >> 1;
    }

    do {                                                                                            // IMPOSTAZIONE ORA ALLARME
        if (Debounce_multi(SW2)) {                                                                  // se  premo SW2
            _hours_al = Inc_pbcd(_hours_al, MIN_HOU, MAX_HOU);                                      // incremento il valore dell'ora dell'allarme
        }

        if (Debounce_multi(SW1)) {                                                                  // se premo SW1
            _hours_al = Dec_pbcd(_hours_al, MIN_HOU, MAX_HOU);                                      // decremento il valore dell'ora dell'allarme
        }

        DelayMs(100);
        GLCD_fill_block(ALARM_TIME_START_X, ALARM_TIME_START_X +42, ALARM_TIME_START_Y -31, ALARM_TIME_START_Y, OFF);
        DelayMs(100);
        GLCD_write_PBCD(ALARM_TIME_START_X, ALARM_TIME_START_Y, _hours_al, FONT_20x31);

    } while (!Debounce_single(SW3));                                                                // ripeto fin che non viene premuto SW3

    do {                                                                                            // IMPOSTAZIONE MINUTI ALLARME
        if (Debounce_multi(SW2)) {
            _minutes_al = Inc_pbcd(_minutes_al, MIN_MIN, MAX_MIN);
        }

        if (Debounce_multi(SW1)) {
            _minutes_al = Dec_pbcd(_minutes_al, MIN_MIN, MAX_MIN);
        }

        DelayMs(100);
        GLCD_fill_block(ALARM_TIME_START_X +56, ALARM_TIME_START_X +98, ALARM_TIME_START_Y -31, TIME_START_Y, OFF);
        DelayMs(100);
        GLCD_write_PBCD(ALARM_TIME_START_X +56, ALARM_TIME_START_Y, _minutes_al, FONT_20x31);

    } while (!Debounce_single(SW3));

                                                                                                    // SELEZIONE GIORNI DELLA SETTIMANA ALLARME
    char _actual = 0;
    char _shift = 0;
    char _incremental = 0;

    for (char _i=1; _i<=64; _i=_i*2) {                                                              // ciclo per ogni bit (ogni bit di peso diverso corrisponde ad 1 giorno)
                                                                                                    // (_i vale ogni volta 1 bit singolo del byte_ 1,2,4,8...)
        _actual = (alarm_by_day >> _shift) & 0b00000001;                                            // leggo il valore attuale per singolo bit (ogni bit corrisponde ad un giorno)

        do {
        if (Debounce_single(SW1) || Debounce_single(SW2)) {                                         // se premo SW1 o SW2
            if (!_actual) {                                                                         // se valore attuale = 0
                alarm_by_day = alarm_by_day + _i;                                                   // aggiungo _i al byta alarm_by_day (dove _i ad ogni ciclo vale il giorno da settare)
                _actual = 1;                                                                        // setto valore attuale
            } else {                                                                                // se valore attuale = 1
                alarm_by_day = alarm_by_day - _i;                                                   // sottraggo _i al byta alarm_by_day dove _i ad ogni ciclo vale il giorno da settare)
                _actual = 0;                                                                        // resetto valore attuale
            }
        }

        if (_actual)                                                                                // se _actual  settato scrivo X nella posizione corrispondente, altrimenti uno spazio vuoto
            GLCD_write_char(ALARM_BY_DAY_START_X + (_incremental*4) + (_incremental*7), ALARM_BY_DAY_START_Y + 9,'X', FONT_7x7);
        else
            GLCD_fill_block(ALARM_BY_DAY_START_X + (_incremental*4) + (_incremental*7), ALARM_BY_DAY_START_X + (_incremental*4) + (_incremental*7) + 6, ALARM_BY_DAY_START_Y +9 -7,ALARM_BY_DAY_START_Y +9,OFF);

        GLCD_drow_line_horizontal(ALARM_BY_DAY_START_X + (_incremental*4) + (_incremental*7),       // attivo una line cursore sotto il giorno da impostare
                                  ALARM_BY_DAY_START_X + (_incremental*4) + (_incremental*7) + 6,
                                  ALARM_BY_DAY_START_Y + 11, ON);

        } while (!Debounce_single(SW3));                                                            // ripeto fin che non viene premuto SW3 per passare algiorno successivo

        GLCD_drow_line_horizontal(ALARM_BY_DAY_START_X + (_incremental*4) + (_incremental*7),       // cancello la linea cursore
                                  ALARM_BY_DAY_START_X + (_incremental*4) + (_incremental*7) + 6,
                                  ALARM_BY_DAY_START_Y + 11, OFF);

        _actual = 0;                                                                                // resetto flag di incremento
        _incremental++;                                                                             // aumento il valore della colonna di scrittura
        _shift++;                                                                                   // aumento indice di shift
    }

    do {                                                                                            // IMPOSTAZIONE ON OFF ALLARME GENERALE
        if (Debounce_multi(SW1) || Debounce_multi(SW2)) {
            alarm = !alarm;
        }

        DelayMs(100);
        GLCD_fill_block(ALARM_ON_X, ALARM_ON_X +14, ALARM_ON_Y -5, ALARM_ON_Y, OFF);
        DelayMs(100);
        if (alarm)
            GLCD_write_string(ALARM_ON_X, ALARM_ON_Y, "ON", FONT_4x5);
        else
            GLCD_write_string(ALARM_ON_X, ALARM_ON_Y, "OFF", FONT_4x5);

    } while (!Debounce_single(SW3));

    do {                                                                                            // IMPOSTAZIONE ON OFF ALLARME GENERALE
        if (Debounce_multi(SW1) || Debounce_multi(SW2)) {
            tone = !tone;
        }

        DelayMs(100);
        GLCD_fill_block(TONE_ON_X, TONE_ON_X +14, TONE_ON_Y -5, TONE_ON_Y, OFF);
        DelayMs(100);
        if (tone)
            GLCD_write_string(TONE_ON_X, TONE_ON_Y, "ON", FONT_4x5);
        else
            GLCD_write_string(TONE_ON_X, TONE_ON_Y, "OFF", FONT_4x5);

    } while (!Debounce_single(SW3));

    RTCC_write_minutes_al(_minutes_al, AL_ON);                                                      // imposto i minuti dell'allarme a abilito l'allarme sui minuti
    RTCC_write_hours_al(_hours_al, AL_ON);                                                          // imposto l'ora dell'allarme a abilito l'allarme sull'ora

    Drow_basic_layout();
    Alarm_check_day(0);                                                                             // abilito l'interrupt a seconda del giorno attuale e giorni di allarme

    T0CONbits.TMR0ON = 1;                                                                           // avvio timer0 per spegnere retroilluminazione dopo 5 secondi

}

/* ============================================================================================================================================= /
 * FUNCTION:    Alarm_check_day
 * Input:       _offset: impostare a 0 se il giorno da verificare  il giorno attuale (arriva da aggiornamento data o da impostazione sveglia)
 *                       impostare a 1 se il giorno da verificare  il successivo (arriva da spegnimento dell'allarme)
 * Overview:    Verifica se il giorno attuale o successivo ha la sveglia e imposta di conseguenza l'interrupt allarme nell'RTCC e il simbolo si sveglia su LCD
/* ============================================================================================================================================ */

void Alarm_check_day(char _offset) {
    char _day_to_check = RTCC_read_day_w() + _offset;                                               // leggo il giorno della settimana dall'RTCC e sommo l'offset
                                                                                                    // (0 verifico se oggi devo attivare l'allarme, 1 verifico se domani devo impostare l'allarme)
    char _alarm_by_day_temp = alarm_by_day;

                                                                                                    // schifto a destra il byte temporaneo contentente i giorni con allarme attivo
    _alarm_by_day_temp = (_alarm_by_day_temp >> _day_to_check)&0b00000001;                          // di tante posizioni qunte il giono da controllare e maschero solo l'ultimo bit

    if (_alarm_by_day_temp && alarm) {                                                              // se il risultato  1 e se attivo l'allarme generale
        RTCC_write_day_w_al(_day_to_check, AL_ON);                                                  // imposto il giorno dell'allarme a abilito l'allarme sul giorno
        RTCC_set_alarm_interrupt(ON);                                                               // abilito l'interrupt dell'allarme da RTCC
        GLCD_write_string(ALARM_WRITE_X, ALARM_WRITE_Y, "ALARM", FONT_4x5);                         // scrivo il simbolo di allarme nella posizione impostata
    } else {                                                                                        // altrimenti
        RTCC_set_alarm_interrupt(OFF);                                                              // disabilito l'interrupt dell'allarme da RTCC
        GLCD_fill_block(ALARM_WRITE_X, ALARM_WRITE_X +24, ALARM_WRITE_Y -5, ALARM_WRITE_Y, OFF);    // e cancello la scritta ALARM
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Inc_pbcd
 * Input:       _value: valore attuale da incrementare
 *              _min: valore minimo consentito del risultato
 *              _max: valore massimo consentito del risultato
 * Output:      Ritorna il valore incrementato di 1 unit
 *
 * Overview:    Incrementa di 1 unit il singolo byte (char) in packed BCD (4bit alti: decine, 4 bit bassi: unit (max 9) mantenendo il risultato
 *              tra il valore minimo e il valore massimo consentito
/* ============================================================================================================================================ */

char Inc_pbcd(char _value, char _min, char _max) {
    _value++;                                                                                       // incremento il valore
    if ((_value & 0b00001111) == 10) _value = _value + 6;                                           // se risultato degli ultimi 4 bit  10 sommo 6 al risultato
    if (_value > _max) _value = _min;                                                               // se risultato supera il valore massimo consentito riporto
                                                                                                    // il risultato al valore minimo consentito
    return(_value);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Dec_pbcd
 * Input:       _value: valore attuale da decrementare
 *              _min: valore minimo consentito del risultato
 *              _max: valore massimo consentito del risultato
 * Output:      Ritorna il valore decrementato di 1 unit
 *
 * Overview:    Decrementa di 1 unit il singolo byte (char) in packed BCD (4bit alti: decine, 4 bit bassi: unit (max 9) mantenendo il risultato
 *              tra il valore minimo e il valore massimo consentito
/* ============================================================================================================================================ */

char Dec_pbcd(char _value, char _min, char _max) {
    _value--;                                                                                       // decremento il valore
    if ((_value & 0b00001111) == 15) _value = _value - 6;                                           // se il rusultato degli ultimi 4 bit  15 sottraggo 6 al risultato
    if (_value < _min || _value > _max) _value = _max;                                              // se il risultato  minore del valore minimo consentito o maggiore
                                                                                                    // del valore massimo (overflow) riporto il risultato al valore massimo
    return(_value);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Update_temp
 *
 * Overview:    legge la temperatura da LM35, ingresso analogico AN2 e scrive la temperatura su LCD
/* ============================================================================================================================================ */

void Update_temp(void) {
    unsigned int _lettura = 0;
    unsigned int _media = 0;
    unsigned int _media_p = 0;
    unsigned int _temp = 0;
    char _string[];

    ADCON0bits.CHS0 = 0;
    ADCON0bits.CHS1 = 1;                                                                            // seleziono AN2 come ingresso
    ADCON0bits.ADON = 1;

    for (char _i =0; _i<64; _i++){                                                                  // ciclo 64 letture per avere una media
        ADCON0bits.GO = 1;                                                                          // avvio acquisizione e conversione AD
        while(ADCON0bits.GO);                                                                       // attendo che venga competata la conversione
        _lettura = ADRESH;                                                                          // salvo il risultato
        _lettura = _lettura << 8;
        _lettura = _lettura + ADRESL;
        _media = _media + _lettura;                                                                 // sommo le varie letture
    }
    ADCON0bits.ADON = 0;
    _media = _media >> 6;                                                                           // divido per 64
    _media = _media - LM35_OFFSET;

    #ifdef FREEDOM
        _media = _media >> 1;
    #endif

    _temp = _media;

    #ifdef DEDICATED_HW
        _string[0] =  _temp/100;                                                                        // divido la lettura in array con i dati da scrivere
        if (_string[0])
            _temp = _temp - (100 * _string[0]);
        _string[1] =  _temp/10;
        if (_string[1])
            _temp = _temp - (10 * _string[1]);
        _string[3]= _temp + 0b00110000;

        _string[0] = _string[0] + 0b00110000;
        _string[1] = _string[1] + 0b00110000;

        _string[2] = '.';
        _string[4] = 0x7F;
        _string[5] = 'C';

        _string[6] = '\0';
    #endif

    #ifdef FREEDOM
        _string[0] =  _temp/10;
        if (_string[0])
            _temp = _temp - (10 * _string[0]);
        _string[1]= _temp + 0b00110000;
        _string[0] = _string[0] + 0b00110000;
        _string[2] = 0x7F;
        _string[3] = 'C';

        _string[4] = '\0';
    #endif


    if ((_media != _media_p) || new_day) {                                                          // se la lettura attuale  diversa dalla precedente, o  un nuovo giorno
        GLCD_fill_block (TEMP_X, 126, TEMP_Y -7, TEMP_Y, OFF);                                      // cancello la scritta precedente
        GLCD_write_string(TEMP_X, TEMP_Y, _string, FONT_7x7);                                       // scrivo su GLCD il dato rilevato
        _media_p = _media;                                                                          // imposto media P
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Check_light
 *
 * Overview:    legge la la luminosita, ingresso analogico AN0 e imposta i valori del ditycile per la retoilluminazione
/* ============================================================================================================================================ */

void Check_light(void) {
    int _lettura = 0;
    char _day_night = 0;
    static char _day_night_p = 0;

    ADCON0bits.CHS0 = 0;                                                                            // seleziono AN0 come ingresso
    ADCON0bits.CHS1 = 0;
    ADCON0bits.ADON = 1;
    ADCON0bits.GO = 1;                                                                              // avvio acquisizione e conversione AD
    while(ADCON0bits.GO)                                                                            // attendo che venga competata la conversione
        continue;
    _lettura = ADRESH;                                                                              // salvo il risultato
    _lettura = _lettura << 8;
    _lettura = _lettura + ADRESL;

    ADCON0bits.ADON = 0;

    if (_lettura > light_treshold) {                                                               // se il dato letto  maggiore della treshold
        _day_night = 1;
        duty_high = DUTY_HIGH_DAY;                                                                 // imposto i valori del duty cicle per il giorno
        duty_low = DUTY_LOW_DAY;
    } else {                                                                                       // altrimenti
        _day_night = 2;                                                                            // imposto i valori del duty cicle per la notte
        duty_high = DUTY_HIGH_NIGHT;
        duty_low = DUTY_LOW_NIGHT;
    }

    if (_day_night != _day_night_p || set_up_treshold) {                                           // aggiorno CCPR2L solo se c' stato un cambio giorno/notte
                                                                                                   // o se arrivo dalla funzione di setup treshold
        CCPR2L = duty_low;                                                                         // con il valore duty_low
        _day_night_p = _day_night;                                                                 // setto _day_night_p
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Set_treshold
 *
 * Overview:    imposta il livello di riferimento per il cambio di intensit della retroilluminazione
/* ============================================================================================================================================ */
void Set_treshold (void) {

    GLCD_write_string (20, 8, "SET UP LIGHT TRESHOLD", FONT_4x5);

    set_up_treshold = 1;
    do {
        if (Debounce_multi(SW2)) {                                                                  // se premo SW2
            light_treshold++;                                                                       // incremento la variabile
        }

        if (Debounce_multi(SW1)) {                                                                  // se premo SW1
            light_treshold--;                                                                       // decremento la variabile
        }

        if (light_treshold > 1023)                                                                  // mantengo la variabile nei limiti
            light_treshold = 0;
        if (light_treshold < 0)
            light_treshold = 1023;

        GLCD_fill_block(TRESHOLD_X, TRESHOLD_X +50, TRESHOLD_Y -7,TRESHOLD_Y , OFF);                // cancello dato precedente
        GLCD_write_unsint(TRESHOLD_X, TRESHOLD_Y, light_treshold, FONT_7x7);                        // scrivo nuovo dato
        DelayMs(500);

        Check_light();                                                                              // imposto il livello della retroilluminazione

    } while (!Debounce_single(SW4));                                                                // ripeto fin che non viene premuto SW4

    set_up_treshold = 0;
    EEPROM_write_int (light_treshold, 0x00);                                                        // salvo l'impostazione in EEPROM
}
