/* ============================================================================================================================================ */
/* LIBRARY:         Lettura e scrittura da EEPROM interna
/* FILE:            EEPROM.c
/* LAST REV:        1.0
/* AUTHOR:          Ivo Colleoni
/* DEVICE:          PIC18F
/* CREATED:         24/09/2013
/* UPDATED:         24/09/2013
/*
/* DESCRIP:         Gestione lettura e scrittura EEPROM interna
/*
/* FUNCTIONS:       - EEPROM_read
/*                  - EEPROM_write
/*                  - EEPROM_read_int
/*                  - EEPROM_write_int
/* ============================================================================================================================================ */
/* Change History:
/* Rev.             Date                Changes
/* 1.0              24/09/2013
/* ============================================================================================================================================ */

/* ============================================================================================================================================ */
/* FUNCTION:        EEPROM_read
/* Input:           _adr: indirizzo da leggere (char)
/* Output           valore letto (char)
/*
/* Overview:        Restituisce il valore contenuto nell'indirizzo dichiarato
/* ============================================================================================================================================ */

char EEPROM_read(char _adr) {
    EEADR = _adr;                                                                       // imposto l'indirizzo
    EECON1bits.EEPGD = 0;
    EECON1bits.CFGS = 0;
    EECON1bits.RD = 1;                                                                  // leggo il dato
    return EEDATA;
}

/* ============================================================================================================================================ */
/* FUNCTION:        EEPROM_write
/* Input:           _data: valore da scrivere (char)
/*                  _adr: indirizzo in cui scrivere (char)
/*
/* Overview:        Salva il valore nell'indirizzo dichiarato
/* ============================================================================================================================================ */

void EEPROM_write(char _data, char _adr) {
   char _flag_interrupt = 0;
   PIR2bits.EEIF = 0;                                                                   // resetto flag di fine scrittura
   EEADR = _adr;                                                                        // imposto indirizzo e dato
   EEDATA = _data;
   EECON1bits.EEPGD = 0;
   EECON1bits.CFGS = 0;
   EECON1bits.WREN = 1;                                                                 // abilito la scrittura

   if (INTCONbits.GIE) {                                                                // se gli interrupt sono abilitati setto a 1 il flag e li disabilito
       _flag_interrupt = 1;
       INTCONbits.GIE =0;
   }
   EECON2 = 0x55;
   EECON2 = 0xAA;
   EECON1bits.WR = 1;                                                                   // scrivo il dato

   while (!PIR2bits.EEIF)                                                               // attendo che il dato sia scritto (EEIF=1)
       continue;
   EECON1bits.WREN = 0;                                                                 // disabilito scrittura

   if (_flag_interrupt) INTCONbits.GIE = 1;                                             // se gli interrupt erano abilitati li riabilito
}

/* ============================================================================================================================================ */
/* FUNCTION:        EEPROM_read_int
/* Input:           _adr: indirizzo in cui scrivere (char)
/* Output           valore letto in _adr e _adr+1 (signed int)
/*
/* Overview:        restituisce un valore intero da -32768 a 32767 letto da 2 indirizzi consecutivi
/* ============================================================================================================================================ */

int EEPROM_read_int(char _adr) {
    int _ret_value;
    char _high;
    char _low;

    _high = EEPROM_read(_adr);                                                          // leggo il byte alto
    _low = EEPROM_read(_adr+1);                                                         // leggo il byte basso

    _ret_value = _high;                                                                 // metto _high nel byte alto di _ret_value
    _ret_value = _ret_value <<8;
    _ret_value = _ret_value + _low;                                                     // aggiungo _low nel byte basso

    return _ret_value;
}

 /* ============================================================================================================================================ */
/* FUNCTION:        EEPROM_write_int
/* Input:           _data: valore da scrivere (signed int)
/*                  _adr: indirizzo in cui scrivere (char)
/*
/* Overview:        Salva il valore intero da -32768 a 32767 nell'indirizzo dichiarato e in quello successivo (2 byte)
/* ============================================================================================================================================ */

void EEPROM_write_int(int _data, char _adr) {
    char _high;
    char _low;

    _low = _data & 0b0000000011111111;                                                  // maschero il byte basso
    _data = _data >>8;                                                                  // sposto il byte alto nel byte basso
    _high = _data;

    EEPROM_write(_high ,_adr);                                                          // scrivo il byte alto
    EEPROM_write(_low, _adr+1);                                                         // scrivo il byte basso
}