/* ============================================================================================================================================= /
 * LIBRARY:         Gestione GLCD
 * FILE:            GLCD_KS0108B.c
 * LAST REV:        1.2
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         03/01/2014
 * UPDATED:         06/02/2014
 *
 * DESCRIP:         Gestione di display GLCD 128*64
 *
 * FUNCTIONS:       PRIVATE FUNCTION
 *                  GLCD_read(char _DI);
 *                  GLCD_send(char _DI, char _data);
 *                  GLCD_set_display(char _status);
 *                  GLCD_set_page(char _page);
 *                  GLCD_set_address(char _address);
 *                  GLCD_wait_busy(void);
 *
 *                  PUBLIC FUNCTION
 *                  GLCD_init(void);
 *                  GLCD_clear(void);
 *                  GLCD_write_byte(char _page, char _address, char _data);
 *                  GLCD_write_bit(char _x, char _y, char _on_off);
 *                  GLCD_drow_image(const char  *_image);
 *                  GLCD_drow_line_horizontal (char _x_origin, char _x_dest, char _y, char _on_off);
 *                  GLCD_drow_line_vertical (char _y_origin, char _y_dest, char _x, char _on_off);
 *                  GLCD_drow_block (char _x_origin, char _x_dest, char _y_origin, char _y_dest, char _on_off);
 *                  GLCD_fill_block (char _x_origin, char _x_dest, char _y_origin, char _y_dest, char _on_off);
 *                  GLCD_write_char(char _x_pos, char _y_pos, char _character, char _font_type);
 *                  GLCD_write_string (char _x_pos, char _y_pos, const char _string[], char _font_type);
 *                  GLCD_write_unsint(char _x_pos, char _y_pos, unsigned int _unsint, char _font_type);
 *                  GLCD_write_sint(char _x_pos, char _y_pos, signed int _sint, char _font_type);
 *                  GLCD_write_PBCD(char _x_pos, char _y_pos,char _pbcd, char _font_type);
 *                  GLCD_set_font_type (char _font_type);
 *                  GLCD_reset_font_type (void);
 *                  GLCD_back_illum(char _on_off);
/* ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 1.2              06/02/2014          Scrittura char in byte e non pi in singoli bit
 *                                      indirizzamento funzioni con valori X da 1 a 128 e Y da 1 a 64
 * 1.1              13/01/2014          Prima release: scrittura immagini, box, caratteri, stringhe, interi, PBCD, font impostabili per singola scritta
 * 1.0              03/01/2014          Inizio studio GLCD
/* ============================================================================================================================================ */

/* INCLUDES =================================================================================================================================== */
#include    "GLCD_KS0108B.h"

/* Variabili globali libreria ================================================================================================================= */
    bit string_mode;                                                                    // indica se si sta scrivendo una stringa (1) o un singolo carattere (0)
    char font_width;                                                                    // parametri del font
    char font_height;
    char font_space;
    char font_direction;
    const char (*font_pointer)[7];                                                      // puntatore all'array del font selezionato
    const char (*font_pointer_2)[22];                                                   // puntatore all'array del font selezionato
    const char (*font_pointer_3)[39];                                                   // puntatore all'array del font selezionato
    const char (*font_pointer_4)[80];                                                   // puntatore all'array del font selezionato

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_read
 * Input:       _DI: indica cosa leggere: STATUS o DATA (char)
 * Output:      ritorna il dato letto (char)
 * Precond.:    Se si vuole leggere un dato dalla memoria ram Settare il CS, la pagina e l'indirizzo corretti.
 *
 * Overview:    Legge il byte di status del GLCD o il dato all'indirizzo settato
/* ============================================================================================================================================ */
char GLCD_read(char _DI) {                                                              //_DI cosa leggere_ 0 status, 1 data
    char _data_readed = 0;

    GLCD_DATA_DIR = INPUT;                                                              // imposto la direzione della port del pic in input

    GLCD_E = LOW;
    __delay_minimo();
    GLCD_DI = _DI;
    GLCD_RW = 1;
    __delay_minimo();
    GLCD_E = HIGH;
    __delay_minimo();

    _data_readed = GLCD_DATA_READ;                                                      // leggo il dato

    __delay_minimo();
    GLCD_E = LOW;

    __delay_minimo();

    GLCD_DI = LOW;
    GLCD_RW = LOW;

    GLCD_DATA_DIR = OUTPUT;
    return (_data_readed);                                                              // ritorna il dato letto
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_send
 * Input:       _DI: indica cosa si sta inviando al GLCD: COMMAND o DATA (char)
 *              _data: dato da inviare (char)
 * Precond.:    Settare il CS, la pagina e l'indirizzo corretti.
 *
 * Overview:    Invia i byte di comando o il byte da scricere all'indirizzo settato
/* ============================================================================================================================================ */
void GLCD_send(char _DI, char _data){

    GLCD_E = LOW;

    __delay_minimo();
    GLCD_DI = _DI;
    GLCD_RW = LOW;
    __delay_minimo();
    GLCD_E = HIGH;

    GLCD_DATA_WRITE = _data;

    __delay_minimo();
    GLCD_E = LOW;
    __delay_minimo();

    GLCD_DI = LOW;
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_set_display
 * Input:       _status: ON o OFF (char)
 *
 * Overview:    Attiva o disattiva il GLCD
/* ============================================================================================================================================ */
void GLCD_set_display(char _status) {

    if (_status == ON) {                                                                // accendo
        GLCD_CS1 = ACTIVE;
        GLCD_CS2 = DEACTIVE;
        GLCD_wait_busy();
        GLCD_send  (COMMAND, GLCD_ON);
        GLCD_wait_busy();

        GLCD_CS1 = DEACTIVE;
        GLCD_CS2 = ACTIVE;
        GLCD_wait_busy();
        GLCD_send  (COMMAND, GLCD_ON);
        GLCD_wait_busy();
    } else {                                                                            // spengo
        GLCD_CS1 = ACTIVE;
        GLCD_CS2 = DEACTIVE;
        GLCD_wait_busy();
        GLCD_send  (COMMAND, GLCD_OFF);
        GLCD_wait_busy();

        GLCD_CS1 = DEACTIVE;
        GLCD_CS2 = ACTIVE;
        GLCD_wait_busy();
        GLCD_send  (COMMAND, GLCD_OFF);
        GLCD_wait_busy();
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_set_page
 * Input:       _page: pagina da selezionare: 0~7 (char)
 * Precond.:    Abilitare il CS corretto
 *
 * Overview:    Seleziona la pagina
/* ============================================================================================================================================ */
void GLCD_set_page(char _page) {
        GLCD_send  (COMMAND, (GLCD_Y_PAGE + _page));                                    // invia il comando per selezionare la pagina + il valore della pagina da selezionare
        GLCD_wait_busy();
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_set_address
 * Input:       _address: indirizzo x da selezionare: 0~127 (char)
 * Precond.:    Abilitare il CS corretto
 *
 * Overview:    Seleziona l'indirizzo
/* ============================================================================================================================================ */
void GLCD_set_address(char _address) {
//        GLCD_wait_busy();
        GLCD_send  (COMMAND, (GLCD_X_ADDRESS + _address));                              // invia il comando per selezionare l'indirizzo + il valore dell'indirizzo
        GLCD_wait_busy();
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_wait_busy
 *
 * Overview:    Attende che il GLCD abbia completato l'istruzione precendente
/* ============================================================================================================================================ */
void GLCD_wait_busy(void){
    char _busy = 1;

    while (_busy == 1) {
        if (GLCD_read (STATUS) & GLCD_BUSY_FLAG)
            _busy = 1;
        else
            _busy = 0;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_init
 *
 * Overview:    Inizializzazione GLCD
/* ============================================================================================================================================ */
void GLCD_init(void){
    GLCD_DATA_DIR = OUTPUT;                                                             // direzione port PIC OUTPUT
    GLCD_RST = GLCD_EN_RESET;                                                           // abilito il reset del GLCD

    GLCD_back_illum(OFF);                                                               // spengo retroilluminazione
    GLCD_E = HIGH;                                                                      // imposto i valori di partenza
    GLCD_RW = HIGH;
    GLCD_DI = LOW;
    GLCD_CS1 = DEACTIVE;
    GLCD_CS2 = DEACTIVE;

    DelayMs (100);

    GLCD_RST = GLCD_DIS_RESET;                                                          // disabilito il reset
    GLCD_clear();                                                                       // cancello tutti i dati in ram
    GLCD_set_display(ON);                                                               // attivo il GLCD
    GLCD_back_illum(ON);                                                                // accendo retroilluminazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_clear
 *
 * Overview:    Cancella tutti i dati scritti sul GLCD
/* ============================================================================================================================================ */
void GLCD_clear(void){
    char _p = 0;
    char _a = 0;

    GLCD_CS1 = ACTIVE;                                                                  // primo quadrante
    GLCD_CS2 = DEACTIVE;
    for (_p=0; _p<MAX_PAGES; _p++){                                                     // invio su tutte le pagine
        GLCD_set_page (_p);
        for (_a=0; _a<MAX_X; _a++){                                                     // a tutti gli indirizzi
            GLCD_set_address(_a);
            GLCD_send (DATA, OFF);                                                      // OFF
        }
    }

    _p = 0;
    _a = 0;

    GLCD_CS1 = DEACTIVE;                                                                // secondo quadrante
    GLCD_CS2 = ACTIVE;
    for (_p=0; _p<MAX_PAGES; _p++){
        GLCD_set_page (_p);
        for (_a=0; _a<MAX_X; _a++){
            GLCD_set_address(_a);
            GLCD_send (DATA, OFF);
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_byte
 * Input:       _page: pagina da selezionare: 0~7 (char)
 *              _address_x: indirizzo x: 0~127 (char)
 *              _data: byte da scrivere (char)
 *
 * Overview:    Scrive un byte nella pagina selezionata all'indirizzo x selezionato preservando eventuali bit da non sovrascrivere (verifica utilit)
 *              Utilizzato in GLCD_write_char
/* ============================================================================================================================================ */
void GLCD_write_byte(char _page, char _address_x, char _data){
    char _new_data;
    char _readed_data;

    if (_address_x < X_RESOLUTION && _page < MAX_PAGES) {                               // se x e y sono nei limiti di risoluzione
        if (_address_x<MAX_X) {                                                         // da 0 a X_MAX (0~63)
            GLCD_CS1 = ACTIVE;                                                          // abilito CS1
            GLCD_CS2 = DEACTIVE;
        } else {                                                                        // altrimenti (64~127)
            GLCD_CS1 = DEACTIVE;                                                        // abilito CS2
            GLCD_CS2 = ACTIVE;
            _address_x = _address_x - MAX_X;                                            // e imposto l' _address_x corretto
        }

        GLCD_set_page (_page);                                                          // setto pagina
        GLCD_set_address (_address_x);                                                  // e indirizzo
        _readed_data = GLCD_read (DATA);                                                // leggo il dato attuale
        _readed_data = GLCD_read (DATA);

        _new_data = _readed_data | _data;                                               // modifico il dato da inviare in modo da non sovrascrivere
                                                                                        // i bit sulla stessa pagina che sono gi attivi per altre scritture
        GLCD_set_address (_address_x);                                                  // reimposto l'indirizzo
        GLCD_send  (DATA, _new_data);                                                   // invio il dato
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_write_bit
 * Input:       _x: indirizzo x: 1~128 (char)
 *              _y: indirizzo y: 1~64 (char)
 *              _on_off: bit acceso o spento: ON o OFF (char)
 *
 * Overview:    Scrive un bit nell'indirizzo xy selezionato (legge il byte alla pagina che comprende _y all'indirizzo _x, modifica il singolo bit
 *              e scrive il byte alla stessa pagina indirizzo _x)
/* ============================================================================================================================================ */
void GLCD_write_bit(char _x, char _y, char _on_off) {
    char _page = 0;
    char _readed_data = 0;
    char _shift = 0;
    char _bit = 0;
    char _new_data = 0;

    _x--;                                                                               // adatto _x per avere valori da 0 a 127
    _y--;                                                                               // adatto _y per avere valori da 0 a 63

    if (_x < X_RESOLUTION && _y < Y_RESOLUTION) {                                       // se x e y sono nei limiti di risoluzione
        _page = _y >> 3;                                                                // ricavo la pagina escludendo i 3 bit bassi di _y

        if (_x<MAX_X) {
            GLCD_CS1 = ACTIVE;                                                          // da 0 a X_MAX (0~63)
            GLCD_CS2 = DEACTIVE;                                                        // abilito CS1
        } else {                                                                        // altrimenti (64~127)
            GLCD_CS1 = DEACTIVE;                                                        // abilito CS2
            GLCD_CS2 = ACTIVE;
            _x = _x - 64;                                                               // e imposto l' _address corretto
        }

        GLCD_set_page (_page);                                                          // setto pagina e indirizzo
        GLCD_set_address (_x);
        _readed_data = GLCD_read (DATA);                                                // leggo il dato attuale
        _readed_data = GLCD_read (DATA);

        _shift = (_y & 0b00000111);                                                     // controllo quale bit devo modificare
        _bit = 0b00000001 << _shift;

        if (_on_off == ON)                                                              // se devo accendere il pixel
            _new_data = _readed_data | _bit;                                            // ricavo il nuovo valore del byte aggiungendo il bit del pixel da accendere
        else
            _new_data = _readed_data & (~ _bit);                                        // altrimenti togliendo il bit del pixel da spegnere



        GLCD_set_address (_x);                                                          // reimposto l'indirizzo
        GLCD_send (DATA, _new_data);                                                    // mando il nuovo byte da scrivere
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_drow_image
 * Input:       _image: puntatore al primo indirizzo in flash dell'array contentente l'immagine
 *
 * Overview:    Scrive un immagine salvata in array sul GLCD
/* ============================================================================================================================================ */
void GLCD_drow_image(const char  * _image) {
    char _p = 0;
    char _a = 0;

    for (_p=0; _p<MAX_PAGES; _p++) {                                                    // per ogni pagina
        GLCD_CS1 = ACTIVE;                                                              // abilito CS1
        GLCD_CS2 = DEACTIVE;

        GLCD_set_address(_a);                                                           // setto indirizzo e pagina
        GLCD_set_page(_p);

        for (_a=0; _a<MAX_X; _a++) {                                                    // per ogni x
            GLCD_send (DATA, *_image);                                                  // invio il dato contenuto all'indirizzo dell'array
            _image++;
        }

        _a = 0;
        GLCD_CS1 = DEACTIVE;                                                            // abilito CS2
        GLCD_CS2 = ACTIVE;

        GLCD_set_address(_a);                                                           // setto indirizzo e pagina
        GLCD_set_page(_p);

        for (_a=0; _a<MAX_X; _a++) {                                                    // per ogni x
            GLCD_send (DATA, *_image);                                                  // invio il dato contenuto all'indirizzo dell'array
            _image++;
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_drow_line_horizontal
 * Input:       _x_origin: x di partenza: 1~128 (char)
 *              _x_dest: x destinazione: 1~128 (char)
 *              _y: indirizzo y: 1~64 (char)
 *              _on_off: accende o spegne i pixel: ON o OFF (char)
 *
 * Overview:    Disegna una linea orizzontale su GLCD da punto _x_origin a punto _x_dest compresi su riga _y
/* ============================================================================================================================================ */
void GLCD_drow_line_horizontal(char _x_origin, char _x_dest, char _y, char _on_off) {
    char _i = 0;

        do {
            GLCD_write_bit ((_x_origin + _i), _y, _on_off);                             // scrivo 1 bit a _x + lo spostamento a destra, y
            _i++;
        } while ((_x_origin + _i)<=_x_dest);                                            // fino a quando raggiungo x destinazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_drow_line_vertical
 * Input:       _y_origin: y di partenza: 1~64 (char)
 *              _y_dest: y destinazione: 1~64 (char)
 *              _x: indirizzo x: 1~128 (char)
 *              _on_off: accende o spegne i pixel: ON o OFF (char)
 *
 * Overview:    Disegna una linea overticale su GLCD da punto _y_origin a punto _y_dest compresi su colonna _x
/* ============================================================================================================================================ */
void GLCD_drow_line_vertical (char _y_origin, char _y_dest, char _x, char _on_off) {
    char _i = 0;

    do {
            GLCD_write_bit (_x, (_y_origin + _i), _on_off);                             // scrivo 1 bit a _y + lo spostamento in basso, x
            _i++;
        } while ((_y_origin + _i)<=_y_dest);                                            // fino a quando raggiungo x destinazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_drow_block
 * Input:       _x_origin: x di partenza: 1~128 (char)
 *              _x_dest: x destinazione: 1~128 (char)
 *              _y_origin: y di partenza: 1~64 (char)
 *              _y_dest: y destinazione: 1~64 (char)
 *              _on_off: accende o spegne i pixel: ON o OFF (char)
 *
 * Overview:    Disegna una rettangolo su GLCD
/* ============================================================================================================================================ */
void GLCD_drow_block (char _x_origin, char _x_dest, char _y_origin, char _y_dest, char _on_off) {

        GLCD_drow_line_horizontal (_x_origin, _x_dest, _y_origin, _on_off);
        GLCD_drow_line_horizontal (_x_origin, _x_dest, _y_dest, _on_off);
        GLCD_drow_line_vertical (_y_origin, _y_dest, _x_origin, _on_off);
        GLCD_drow_line_vertical (_y_origin, _y_dest, _x_dest, _on_off);
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_fill_block
 * Input:       _x_origin: x di partenza: 1~128 (char)
 *              _x_dest: x destinazione: 1~128 (char)
 *              _y_origin: y di partenza: 1~64 (char)
 *              _y_dest: y destinazione: 1~64 (char)
 *              _on_off: accende o spegne i pixel: ON o OFF (char)
 *
 * Overview:    Disegna un rettangolo pieno su GLCD compreso tra i punti x y indicati (punti compresi)
/* ============================================================================================================================================ */
void GLCD_fill_block (char _x_origin, char _x_dest, char _y_origin, char _y_dest, char _on_off) {
    char _i = 0;
    do {
        GLCD_drow_line_horizontal (_x_origin, _x_dest, (_y_origin + _i), _on_off);
        _i++;
    } while ((_y_origin + _i)<=_y_dest);                                                // fino a quando raggiungo y destinazione

}

/* ============================================================================================================================================= /
 * FUNCTION:     GLCD_write_char
 * Input:        _x_pos: x di partenza: 1~128, pixel pi basso della scritta (char)
 *               _y_pos: y di partenz: 1~64, pixel pi a sinistra della scritta (char)
 *               _character: carettere da scrivere (char)
 *               _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 * Output:       ritorna il valore di address(x) effettivamente scritti
 *
 * Overview:    Scrive un singlo carattere su GLCD, se string_mode == 1 i parametri del font non vengono letti.
 *              - scrittura font con array a byte verticali: elimina le colonne vuote in ogni byte carattere del font
 *              - gestione separata font a 1, 2, 3 e 4 byte verticali (per i font a 4 byte vengono gestiti solamente i caratteri numerici) e /:.
 *              - la scrittura su GLCD viene fatta per pagina e non per bit attraverso la funzione GLCD_write_byte per velocizzare la scrittura.
/* ============================================================================================================================================ */
char GLCD_write_char(char _x_pos, char _y_pos, char _character, char _font_type) {      //indicare pozizione bit piu basso
    char _data_0 = 0;
    char _data_1 = 0;
    char _data_2 = 0;
    char _data_3 = 0;
    char _i = 0;
    char _effective_x = 0;
    char _return = 0;
    char _page = 0;
    char _page_max = 0;
    char _offset = 0;
    char _byte_0 = 0;
    char _byte_1 = 0;
    char _byte_2 = 0;
    char _byte_3 = 0;
    char _byte_4 = 0;


    if (string_mode == OFF) {                                                           // se non sono in string mode
        GLCD_set_font_type(_font_type);                                                 // imposto i parametri del font
    }

    _y_pos--;                                                                           // adattp x e y per avere valori che partono da 0
    _x_pos--;

    _page_max = _y_pos >> 3;                                                            // ricavo la pagina pi alta (piu in basso sul display)
    _y_pos = (_y_pos +1 - font_height);                                                 // ricavo il primo bit che dovr scrivere
    _page = _y_pos >> 3;                                                                // ricavo la pagina pi bassa (piu in alto sul display)
    _offset = _y_pos & 0b00000111;                                                      // ricavo l'offset da aggiungere al primo byte (primo bit del byte da scrivere)

/*  SCRITTURA FONT A 1 BYTE VERTICALE  */
    #ifdef DIRECTION_D1                                                                 // se utilizzo almeno 1 font a direzione verticale 1 byte compilo questa parte di programma
    if (font_direction == 0xD1) {                                                       // font con array a 1 byte verticali
       for (_i=0; _i<font_width; _i++) {                                                // ciclo per N colonne
            _data_0 = font_pointer[_character-31][_i];                                  // leggo la colonna selezionata per il carattere selezionato dall'array del font (offset 31 rispetto alla tabella ASCII standard)
            if (_data_0 != 0) {                                                         // se il byte letto = 0 non scrivo nulla
                if (_page == _page_max) {                                               // se la pagina minore e la pagina maggiore hanno setto valore devo scrivere sulla stessa pagina
                    _byte_0 = _data_0 << _offset;                                       // ricavo il dato da scrivere tremite shift del dato letto

                    GLCD_write_byte(_page, _x_pos + _effective_x, _byte_0);             // scrivo il byte
                } else {                                                                // altrimenti devo scrivere il dato su 2 pagine
                    _byte_1 = _data_0 >> (8-_offset);                                   // ricavo i 2 byte da scrivere tramite shift del dato letto
                    _byte_0 = _data_0 << _offset;

                    GLCD_write_byte(_page, _x_pos + _effective_x, _byte_0);             // scrivo il byte alla pagina pi alta
                    GLCD_write_byte((_page +1), _x_pos + _effective_x, _byte_1);        // scrivo il byte alla pagina seguente
                }
                _effective_x++;                                                         // se c' stata scrittura della colonna incremento il contatore delle colonne scritte
            }
        }
        _return = (_effective_x + font_space);                                          // torno il valore effettivo di _x + lo spazio tra i caratteri
    }
    #endif

/*  SCRITTURA FONT A 2 BYTE VERTICALI  */
    #ifdef DIRECTION_D2                                                                 // se utilizzo almeno 1 font a direzione verticale 2 byte compilo questa parte di programma
    if (font_direction == 0xD2){                                                        // font con array a 2 byte verticali
        for (_i=0; _i<font_width*2; _i = _i + 2) {                                      // ciclo per ogni colonna (da 2 byte}
            _data_0 = font_pointer_2[_character-31][_i];                                // leggo i 2 byte consecutivi che fornamo la colonna selezionata del carattere (offset 31 rispetto alla tabella ASCII standard)
            _data_1 = font_pointer_2[_character-31][_i+1];

            if ((_data_0 || _data_1) != 0) {                                            // se nessuno dei dati letti ha valore non scrivo nulla
                if (_page +1 == _page_max) {                                            // se la pagina minore e la pagina maggiore sono consecutive devo scrivere solo in 2 pagine
                    _byte_1 = (_data_1 << _offset) + (_data_0 >> (8-_offset));          // ricavo i 2 byte da scrivere tramite shift del dato letto
                    _byte_0 =  _data_0 << _offset;

                    GLCD_write_byte(_page, _x_pos + _effective_x, _byte_0);             // scrivo il byte alla pagina pi alta
                    GLCD_write_byte((_page +1), _x_pos + _effective_x, _byte_1);        // scrivo il byte alla pagina seguente
                } else {                                                                // altrimenti devo scrivere il dato su 3 pagine
                    _byte_2 = _data_1 >> (8-_offset);                                   // ricavo i 3 byte da scrivere tramite shift del dato letto
                    _byte_1 = (_data_1 << _offset) + (_data_0 >> (8-_offset));
                    _byte_0 =  _data_0 << _offset;

                    GLCD_write_byte(_page, _x_pos + _effective_x, _byte_0);             // scrivo i 3 byte sulle 3 pagine consecutive
                    GLCD_write_byte((_page +1), _x_pos + _effective_x, _byte_1);
                    GLCD_write_byte((_page +2), _x_pos + _effective_x, _byte_2);
                }
                _effective_x++;                                                         // se c' stata scrittura della colonna incremento il contatore delle colonne scritte
            }
        }
        _return = (_effective_x + font_space);                                          // torno il valore effettivo di _x + lo spazio tra i caratteri
    }
    #endif

/*  SCRITTURA FONT A 3 BYTE VERTICALI  */
    #ifdef DIRECTION_D3                                                                 // se utilizzo almeno 1 font a direzione verticale 3 byte compilo questa parte di programma
    if (font_direction == 0xD3){                                                        // font con array a 3 byte verticali

        for (_i=0; _i<font_width*3; _i = _i + 3) {                                      // ciclo per ogni colonna (da 3 byte}
            _data_0 = font_pointer_3[_character-31][_i];                                // leggo i 3 byte consecutivi che fornamo la colonna selezionata del carattere (offset 31 rispetto alla tabella ASCII standard)
            _data_1 = font_pointer_3[_character-31][_i+1];
            _data_2 = font_pointer_3[_character-31][_i+2];

                if ((_data_0 || _data_1 || _data_2) != 0) {                             // se nessuno dei dati letti ha valore non scrivo nulla
                    if (_page +2 == _page_max) {                                        // se devo scrivere in 3 pagine
                        _byte_2 = (_data_2 << _offset) + (_data_1 >> (8-_offset));      // ricavo i 3 byte da scrivere tramite shift del dato letto
                        _byte_1 = (_data_1 << _offset) + (_data_0 >> (8-_offset));
                        _byte_0 =  _data_0 << _offset;

                        GLCD_write_byte(_page, _x_pos + _effective_x, _byte_0);         // scrivo i 3 byte sulle 3 pagine consecutive
                        GLCD_write_byte((_page +1), _x_pos + _effective_x, _byte_1);
                        GLCD_write_byte((_page +2), _x_pos + _effective_x, _byte_2);
                    } else {                                                            // altrimenti devo scrivere su 4 pagine
                        _byte_3 = _data_2 >> (8-_offset);                               // ricavo i 4 byte da scrivere tramite shift del dato letto
                        _byte_2 = (_data_2 << _offset) + (_data_1 >> (8-_offset));
                        _byte_1 = (_data_1 << _offset) + (_data_0 >> (8-_offset));
                        _byte_0 =  _data_0 << _offset;

                        GLCD_write_byte(_page, _x_pos + _effective_x, _byte_0);         // scrivo i 4 byte sulle 4 pagine consecutive
                        GLCD_write_byte((_page +1), _x_pos + _effective_x, _byte_1);
                        GLCD_write_byte((_page +2), _x_pos + _effective_x, _byte_2);
                        GLCD_write_byte((_page +3), _x_pos + _effective_x, _byte_3);
                    }

                _effective_x++;                                                         // se c' stata scrittura della colonna incremento il contatore delle colonne scritte
                }

        }
        _return = (_effective_x + font_space);                                          // torno il valore effettivo di _x + lo spazio tra i caratteri
    }
    #endif

    /*  SCRITTURA FONT A 4 BYTE VERTICALI  */
    #ifdef DIRECTION_D4                                                                 // se utilizzo almeno 1 font a direzione verticale 3 byte compilo questa parte di programma
    if (font_direction == 0xD4){                                                        // font con array a 3 byte verticali

        for (_i=0; _i<font_width*4; _i = _i + 4) {                                      // ciclo per ogni colonna (da 3 byte}
            _data_0 = font_pointer_4[_character-45][_i];                                // leggo i 4 byte consecutivi che fornamo la colonna selezionata del carattere (offset 45 rispetto alla tabella ASCII standard)
            _data_1 = font_pointer_4[_character-45][_i+1];
            _data_2 = font_pointer_4[_character-45][_i+2];
            _data_3 = font_pointer_4[_character-45][_i+3];

//                if ((_data_0 || _data_1 || _data_2 || _data_3) != 0) {                  // se nessuno dei dati letti ha valore non scrivo nulla
                    if (_page +3 == _page_max) {                                        // se devo scrivere in 4 pagine
                        _byte_3 = (_data_3 << _offset) + (_data_2 >> (8-_offset));      // ricavo i 4 byte da scrivere tramite shift del dato letto
                        _byte_2 = (_data_2 << _offset) + (_data_1 >> (8-_offset));
                        _byte_1 = (_data_1 << _offset) + (_data_0 >> (8-_offset));
                        _byte_0 =  _data_0 << _offset;

                        GLCD_write_byte(_page, _x_pos + _effective_x, _byte_0);         // scrivo i 4 byte sulle 4 pagine consecutive
                        GLCD_write_byte((_page +1), _x_pos + _effective_x, _byte_1);
                        GLCD_write_byte((_page +2), _x_pos + _effective_x, _byte_2);
                        GLCD_write_byte((_page +3), _x_pos + _effective_x, _byte_3);
                    } else {                                                            // altrimenti devo scrivere su 5 pagine
                        _byte_4 = _data_3 >> (8-_offset);                               // ricavo i 5 byte da scrivere tramite shift del dato letto
                        _byte_3 = (_data_3 << _offset) + (_data_2 >> (8-_offset));
                        _byte_2 = (_data_2 << _offset) + (_data_1 >> (8-_offset));
                        _byte_1 = (_data_1 << _offset) + (_data_0 >> (8-_offset));
                        _byte_0 =  _data_0 << _offset;

                        GLCD_write_byte(_page, _x_pos + _effective_x, _byte_0);         // scrivo i 5 byte sulle 5 pagine consecutive
                        GLCD_write_byte((_page +1), _x_pos + _effective_x, _byte_1);
                        GLCD_write_byte((_page +2), _x_pos + _effective_x, _byte_2);
                        GLCD_write_byte((_page +3), _x_pos + _effective_x, _byte_3);
                        GLCD_write_byte((_page +4), _x_pos + _effective_x, _byte_4);
                    }

                _effective_x++;                                                         // se c' stata scrittura della colonna incremento il contatore delle colonne scritte
//                }

        }
        _return = (font_width + font_space);                                            // torno il valore effettivo di _x + lo spazio tra i caratteri
    }
    #endif

    return (_return);
}

/* ============================================================================================================================================= /
 * FUNCTION:     GLCD_write_string
 * Input:        _x_pos: x di partenza: 1~128, punto pi basso della scritta (char)
 *               _y_pos: y di partenz: 1~64, punto pi basso della scritta (char)
 *               _string: puntatore ad array di caratteri da scrivere, byte finale \0
 *               _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 *
 * Overview:    Scrive una stringa di caratteri su GLCD
 *              per font con array a byte verticali lo spazio viene codificato con un incremento di _x_pos proporzionale all'ampliezza del font
 *              (per permettere la scrittura salva spazio)
/* ============================================================================================================================================ */
void GLCD_write_string (char _x_pos, char _y_pos, const char _string[], char _font_type) {
    char _i = 0;
    static char _incremental_x_pos = 0;                                                 // variabile statica per il conteggio delle x effettivamente scriti

    string_mode = ON;                                                                   // setto la modalita scrittura stringa
    GLCD_set_font_type(_font_type);                                                     // leggo e imposto i parametri del font

        while (_string[_i]!='\0') {                                                     // per ogni carattere della stringa fino a carattere nullo
            if (_string[_i] == ' ') {                                                   // se carattere spazio
                    _incremental_x_pos = _incremental_x_pos + (3 * font_space);         // incremento la posizione x proporzionalmente a font_space
            } else {                                                                    // altrimenti scrivo il carattere e aumento di conseguenza la posizione x
                _incremental_x_pos = _incremental_x_pos + (GLCD_write_char((_x_pos + _incremental_x_pos), _y_pos, _string[_i], _font_type));
            }
        _i++;
        }

    _incremental_x_pos = 0;                                                             // a fine stringa resetto la posizione di x
    
    string_mode = OFF;                                                                  // disabilitop modalita stringa
    GLCD_reset_font_type();                                                             // resetto i parametri del font
}

/* ============================================================================================================================================= /
 * FUNCTION:     GLCD_write_unsint
 * Input:        _x_pos: x di partenza: 1~128, punto pi basso della scritta (char)
 *               _y_pos: y di partenz: 1~64, punto pi basso della scritta (char)
 *               _unsint: valore numerico unsigned int
 *               _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 *
 * Overview:    Scrive un numero unsigned int su GLCD dopo averlo convertuto in array stringa di caratteri ASCII
/* ============================================================================================================================================ */
void GLCD_write_unsint(char _x_pos, char _y_pos, unsigned int _unsint, char _font_type) {
    char _i = 0;
    char _z = 0;
    char _c = 0;
    unsigned int _denom = 0;
    char _converted[6] = {0,0,0,0,0,0};
    char _converted_format[6] = {0,0,0,0,0,0};

    for (_i=0; _i<4; _i++) {                                                            // converto unsigned int
        if (_i==0) {_denom = 10000;}                                                    
        if (_i==1) {_denom = 1000;}                                                     
        if (_i==2) {_denom = 100;}                                                      
        if (_i==3) {_denom = 10;}                                                       

         _converted[_i] = _unsint/_denom;                                               // in array da 5 caratteri
        if (_converted[_i]) {                                                           // se risultato != 0
            _unsint = _unsint - (_denom * _converted[_i]);                              // sottraggo il valore gi scritto
        }
    }
    _converted[4]= _unsint;                                                             // riporto il resto nell'array
    _converted[5] = '\0';                                                               // aggiungo il carattere di fine stringa

    _i = 0;
    _z = 0;

    while (_converted[_i] == 0 && _i < 4) {                                             // conto quanti 0 ho prima della prima cifra significativa, massimo 4 cifre, l'ultima pu essere 0
        _i++;
    }

   _z = (5 - _i);                                                                       // calcolo la dimensione del nuovo array
  
    for (_c = 0; _c < _z; _c++) {                                                       // ciclo 5 volte - il numero di 0 contati all'inizio del dato
        _converted_format[_c] = _converted[_i] + 0b00110000;                            // salvo il nuovo array da 0 a _z e aggiunco i caratteri per ricavare il codice ASCII del numero
        _i++;
    }
    _converted_format[_c] = '\0';                                                       // aggiungo valore di fine stringa

    GLCD_write_string(_x_pos, _y_pos, _converted_format, _font_type);                   // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:     GLCD_write_sint
 * Input:        _x_pos: x di partenza: 1~128, punto pi basso della scritta (char)
 *               _y_pos: y di partenz: 1~64, punto pi basso della scritta (char)
 *               _sint: valore numerico signed int
 *               _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 *
 * Overview:    Scrive un numero signed int su GLCD dopo averlo convertuto in array stringa di caratteri ASCII
/* ============================================================================================================================================ */
void GLCD_write_sint (char _x_pos, char _y_pos, signed int _sint, char _font_type) {
    if (_sint <0) {                                                                     // se < 0
        GLCD_write_char(_x_pos, _y_pos, '-', _font_type);                               // scrivo il segno -
        GLCD_write_unsint((_x_pos + font_width), _y_pos, ~(_sint-1), _font_type);       // scrivo sul display il complemento a 1 + 1
    } else {                                                                            // se positivo
         GLCD_write_unsint(_x_pos, _y_pos, _sint, _font_type);                          // tratto come se fosse un unsigned int
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:     GLCD_write_PBCD
 * Input:        _x_pos: x di partenza: 1~128, punto pi basso della scritta (char)
 *               _y_pos: y di partenz: 1~64, punto pi basso della scritta (char)
 *               _pbcd: valore numerico codificato in packed BCD
 *               _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 *
 * Overview:    Scrive un numero codificado in 1 byte packed BCD su GLCD (non salva spazio)
/* ============================================================================================================================================ */
void GLCD_write_PBCD(char _x_pos, char _y_pos,char _pbcd, char _font_type) {
    char _H = 0;                                                                        // decine
    char _L = 0;                                                                        // unit
    char _second_x = 0;

    _L = _pbcd & 0b00001111;                                                            // unit = 4 bit bassi del byte in ingresso
    _H = (_pbcd & 0b11110000) >> 4;                                                     // decine = 4 bit alti del dato in ingresso

    _second_x = (GLCD_write_char(_x_pos, _y_pos, (_H + 0b00110000), _font_type));       // scrivo decine (codifica numero asci) e salvo il valore delle x effettivamente scritte
    GLCD_write_char(_x_pos+_second_x, _y_pos, (_L + 0b00110000), _font_type);           // scrivo unit (codifica numero asci)
}

/* ============================================================================================================================================= /
 * FUNCTION:     GLCD_set_font_type
 * Input:        _font type: costante che identifica il tipo di font, dichiarata in ogni include dei singli font
 *
 * Overview:    Imposta le variabili a seconda del font selezionato
/* ============================================================================================================================================ */
void GLCD_set_font_type (char _font_type) {
    // _font_type:  0:  4x5 1 byte verticale
    //              1:  5x7 1 byte verticale
    //              2:  7x7 1 byte verticale
    //            3-9:  liberi per 1 byte verticale
    //             10:  9x15 2 byte verticali
    //          11-19:  liberi per 2 byte verticale
    //             20:  12x19 3 byte verticali
    //             21:  13x24 3 byte verticali
    //          22-29:  liberi per 3 byte verticali
    //             30:  18x26 4 byte verticali solo numeri .:/ (ASCII 46-58
    //             31:  20x31 4 byte verticali solo numeri .:/ (ASCII 46-58

    font_pointer = 0;
    font_pointer_2 = 0;
    font_pointer_3 = 0;
    font_pointer_4 = 0;

    switch (_font_type) {
        #ifdef FONT_4x5_ENABLED                                                         // se FONT_4x5  utilizzato compilo questa parte di programma
        case FONT_4x5: font_pointer = font_4x5;                                         // se FONT_4x5 (case 0)  selezionato imposto il puntatore all'array del font
                break;
        #endif
        #ifdef FONT_5x7_ENABLED
        case FONT_5x7: font_pointer = font_5x7;
                break;
        #endif
        #ifdef FONT_7x7_ENABLED
        case FONT_7x7: font_pointer = font_7x7;
                break;
        #endif
        #ifdef FONT_9x15_ENABLED
        case FONT_9x15: font_pointer_2 = font_9x15;
                break;
        #endif
        #ifdef FONT_12x19_ENABLED
        case FONT_12x19: font_pointer_3 = font_12x19;
                break;
        #endif
        #ifdef FONT_13x24_ENABLED
        case FONT_13x24: font_pointer_3 = font_13x24;
                break;
        #endif
        #ifdef FONT_18x26_ENABLED
        case FONT_18x26: font_pointer_4 = font_18x26;
                break;
        #endif
        #ifdef FONT_20x31_ENABLED
        case FONT_20x31: font_pointer_4 = font_20x31;
                break;
        #endif

    }
                                                                                        // leggo i paarametiri del font dall'array e li salvo nelle variabili globali
    if (_font_type < 10) {                                                              // font a 1 byte
        font_width = font_pointer[0][0];
        font_height = font_pointer[0][1];
        font_space = font_pointer[0][2];
        font_direction = font_pointer[0][4];
    }
    if (_font_type >= 10 &&  _font_type < 20) {                                         // font a 2 byte
        font_width = font_pointer_2[0][0];
        font_height = font_pointer_2[0][1];
        font_space = font_pointer_2[0][2];
        font_direction = font_pointer_2[0][4];
    }

    if (_font_type >= 20 && _font_type < 30) {                                          // font a 3 byte
        font_width = font_pointer_3[0][0];
        font_height = font_pointer_3[0][1];
        font_space = font_pointer_3[0][2];
        font_direction = font_pointer_3[0][4];
    }
    if (_font_type >= 30 && _font_type < 40) {                                          // font  4 byte
        font_width = font_pointer_4[0][0];
        font_height = font_pointer_4[0][1];
        font_space = font_pointer_4[0][2];
        font_direction = font_pointer_4[0][4];
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:     GLCD_reset_font_type
 *
 * Overview:    resetta le variabili a seconda del font selezionato
/* ============================================================================================================================================ */
void GLCD_reset_font_type(void) {
//    string_mode = 0;
    font_width = 0;
    font_height = 0;
    font_space = 0;
    font_direction = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    GLCD_back_illum
 * Input:       _on_off: ON o OFF (char)
 *
 * Overview:    Attiva o disattiva la retroilluminazione
/* ============================================================================================================================================ */
void GLCD_back_illum(char _on_off){
    GLCD_BACK_ILL = _on_off;
}