/* ============================================================================================================================================= /
 * LIBRARY:         Gestione Input
 * FILE:            Manage_input.c
 * LAST REV:        1.2
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         12/09/2013
 * UPDATED:         02/12/2013
 *
 * DESCRIP:         Gestione degli input (fino a 8 tasti conteporaneamente), utilizzare testi sulla stessa PORT
 *
 * FUNCTIONS:       - Debounce_multi
 *                  - Debounce_sigle
 *                  - Read_keypad
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 1.2              03/12/2013          Inserite costati di configurazione ritardi
 * 1.1              27/09/2013          Inserita costante PRESSED per indicare se i pulsanti chiudono a GND o a VCC
 * 1.0              12/09/2013
/* ============================================================================================================================================ */

/* INCLUDES =================================================================================================================================== */
#include "Manage_input.h"

/* ============================================================================================================================================= /
 * FUNCTION         Debounce_multi
 * Conditions:      dichiarare il valore di N: numero di pressioni continue prima di passare all'incremento veloce
 *                  dichiarare PRESSED: 0 se i pulsanti chiudono a GND 1 se i pulsanti chiudono a VCC
 * Input:           *_address: indirizzo della PORT dei tasti, _mask: maschera per lettura bit dello switch. es: &PORTA,1 o mnemonico
 * Output:          1: tasto premuto, 0: tasto non premuto
 *
 * Overview:        se il pulsante che si trova sull'ingresso indicato in _mask sulla porta indicata in _address  premuto torna 1
 *                  gestione pressione multipla: dopo N pressioni continue passa a pressione rapida
/* ============================================================================================================================================ */

char Debounce_multi(char *_address, char _mask) {
    char _ret = 0;

    static char _pressed[8];                                                            // variabile statica che conta le pressioni consecutive, resta invariata all'uscita della fuznione
    char _array_index = 0;                                                              // indice dell'array
    char _temp = _mask;                                                                 // valore temporaneo di _mask

    while (_temp != 1) {                                                                // creo l'indice dell'array da 0 a 7 contando quanti
        _temp = _temp>>1;                                                               // shift a desra di _temp (_mask) devo fare
        _array_index++;
    }

    if ((*_address & _mask)== PRESSED) {                                                // leggo all'indirizzo contenuto in _address, se uguale a PRESSED
        DelayMs(DELAY_DEB);                                                             // delay
        if ((*_address & _mask)== PRESSED) {                                            // se il tasto  ancora premuto
            _pressed[_array_index]++;                                                   // incrementa contatore per la posiziona array relativa al tasto
            if (_pressed[_array_index] <= N) {                                          // se contatore  minore di N
                DelayMs(DELAY_MULTI_LONG);                                              // ritardo lungo
                _ret = 1;                                                               // e torna 1
            } else {
                _pressed[_array_index]--;                                               // altrimenti mantieni il contatore a N
                DelayMs(DELAY_MULTI_SHORT);                                             // ritardo corto
                _ret = 1;                                                               // e torna
            }
        }
    } else {
        _pressed[_array_index] = 0;                                                     // altrimenti azzero il contatore
        _ret = 0;
    }                                                                                   // e ritorno 0
    return _ret;
}

/* ============================================================================================================================================= /
 * FUNCTION         Debounce_single
 * COnditions:      dichiarare PRESSED: 0 se i pulsanti chiudono a GND 1 se i pulsanti chiudono a VCC
 * Input:           *_address: indirizzo della PORT dei tasti, _mask: maschera per lettura bit dello switch. es: &PORTA,1 o mnemonico
 * Output:          1: tasto premuto, 0: tasto non premuto
 *
 * Overview:        se il pulsante che si trova sull'ingresso indicato in _mask sulla porta indicata in _address  premuto torna 1
 *                  gestione pressione singola (dopo la prima pressione le successive non vengono calcolate fino al rilascio del tasto)
/* ============================================================================================================================================ */

char Debounce_single(char *_address, char _mask) {
    char _ret = 0;

    static char _pressed[8];                                                            // variabile statica che conta le pressioni consecutive, resta invariata all'uscita della fuznione
    char _array_index = 0;                                                              // indice dell'array
    char _temp = _mask;                                                                 // valore temporaneo di _mask

    while (_temp != 1) {                                                                // creo l'indice dell'array da 0 a 7 contando quanti
        _temp = _temp>>1;                                                               // shift a desra di _temp (_mask) devo fare
        _array_index++;
    }

    if ((*_address & _mask)== PRESSED) {                                                // leggo all'indirizzo contenuto in _address, se uguale a PRESSED
        DelayMs(DELAY_DEB);                                                                    // delay
        if ((*_address & _mask)== PRESSED) {                                            // se il tasto  ancora premuto
            _pressed[_array_index]++;                                                   // incrementa contatore per la posiziona array relativa al tasto
            if (_pressed[_array_index]==1) {                                            // se contatore  1 (prima pressione)
                _ret = 1;                                                               // torna 1
            } else {
                _pressed[_array_index]--;                                               //altrimenti mantieni il contatore a 1
                _ret = 0;                                                               //e torna
            }
        }
    } else {
        _pressed[_array_index] = 0;                                                     // altrimenti azzero il contatore
        _ret = 0;
    }                                                                                   // e ritorno 0
    return _ret;
}

/* ============================================================================================================================================= /
 * FUNCTION         Read_Keypad
 * Precond:         Collegamento standard: colonne sui bit da 0 a 3 e righe sui bit da 4 a 7
 *                  definire su quale porta  collegata la tastiera mediante #define:
 *                  #define COLUMN LATx
 *                  #define ROW (PORTx &0b11110000): mascherare solo i bit dedicati alle righe
 *                  se non si utilizza il collegamento standard modificare gli array delle righe e delle colonne come opportuno
 *
 * Output:          0xFF se non  premuto nessun tasto, numeri da 0 a 9 per i tasti corrispondenti, *->A, #->B ....
 *
 * Overview:        Legge da tastiera 4x3 e 4x4 e restituisce il valore del tasto premuto, se non viene premuto nessun tasto viene restituito 0xFF.
 *                  Integra antirimbalzo ed evita la pressione multipla dei tasti (se si tiene premuto un tasto verr letto solamente una volta)
/* ============================================================================================================================================ */

//char Read_keypad (void) {
//    const char  _column[] = {                                                           // Array contentente l'ordine delle colonne da abilitare (sulla singola porta)
//                            0b00000001,
//                            0b00000010,
//                            0b00000100,
//                            0b00001000
//                            };
//
//    const char  _row[] = {
//                         0b00010000,                                                    // Array contentente l'ordine delle righe da testare (sulla singola porta)
//                         0b00100000,
//                         0b01000000,
//                         0b10000000
//                         };
//
//    const char _key[] = {0xFF,1,4,7,0x0A,2,5,8,0,3,6,9,0x0B,0x0C,0x0D,0x0E,0x0F};       // Array con i valori di ritorno (0xFF se non viene premuto nulla)
//
//    char _col_index;
//    char _row_index;
//    char _inc = 0;
//    static char _pressed[16];                                                           // Array statico per contare la pressione singola e non prolungata del tasto
//
//    for (_col_index=0; _col_index<4; _col_index++) {                                    // Ciclo di abilitazione delle colonne
//        COLUMN = _column[_col_index];                                                   // Abilito la colonna
//        for (_row_index=0; _row_index<4; _row_index++) {                                // Ciclo di test delle righe
//            _inc++;                                                                     // Incremento contatore generale
//            if (ROW == _row[_row_index]) {                                              // Se il tasto che sto testando  premuto
//                DelayMs(DELAY_DEB);
//                if (ROW == _row[_row_index]) {                                          // lo ricontrollo dopo 20ms
//                    _pressed[_inc]++;                                                   // se  ancora premuto incremento il contatore di pressioni continue per il tasto
//                    if (_pressed[_inc] == 1){return _key[_inc];}                        // se  la prima volta che  stato premuto ritorno con il valore corretto
//                    else {_pressed[_inc]--;}                                            // altrimenti decremento il contatore di pressioni continue per il tasto
//                }
//            } else {_pressed[_inc] = 0;}                                                  // se il tasto che sto testando non  premuto resetto il contatore di pressioni continue
//        }
//    }
//    return _key[0];                                                                     // se non  stato premuto niente ritorno con il primo valore dell'array
//}