/* ============================================================================================================================================= /
 * LIBRARY:         RTCC PCF8563
 * FILE:            RTCC_PCF8563.c
 * LAST REV:        1.2
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         02/12/2013
 * UPDATED:         05/12/2013
 *
 * DESCRIP:         Gestione real time clock calendar PCF8563
 *
 * PRECONDITIONS:   - inizializzare il protocollo I2C tramite libreria I2C_init.c
 * FUNCTIONS:       - RTCC_read_seconds()
 *                  - RTCC_read_minutes()
 *                  - RTCC_read_hours()
 *                  - RTCC_read_day()
 *                  - RTCC_read_month()
 *                  - RTCC_read_year()
 *                  - RTCC_read_day_w()
 *                  - RTCC_read_minutes_al()
 *                  - RTCC_read_hours_al()
 *                  - RTCC_read_day_al()
 *                  - RTCC_read_day_w_al()
 *                  - RTCC_write_seconds(0xSS)
 *                  - RTCC_write_minutes(0xMM)
 *                  - RTCC_write_hours(0xHH)
 *                  - RTCC_write_day(0xDD)
 *                  - RTCC_write_month(0xMM)
 *                  - RTCC_write_year(0xYY)
 *                  - RTCC_write_day_w0x0D)
 *                  - RTCC_write_minutes_al(0xMM, AL_ON)
 *                  - RTCC_write_hours_al(0xHH, AL_ON)
 *                  - RTCC_write_day_al(0xDD, AL_ON)
 *                  - RTCC_write_day_w_al(0x0D, AL_ON)
 *                  - RTCC_set_alarm_interrupt(ON)
 *                  - RTCC_clear_alarm()
 *                  - RTCC_clock_out(1_Hz)
 *
/* ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 1.2              05/12/2013          - Aggiunta gestione clock out
 * 1.1              03/12/2013          - Gestiti registri allarme
 * 1.0              02/12/2013          - Prima release: lettura e scrittura data, ora, giorno
/* ============================================================================================================================================ */

/* INCLUDES =================================================================================================================================== */
#include    "RTCC_PCF8563.h"
#include    <i2c.h>

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_seconds
 * Output:      torna il valore dei secondi codificato in packed BCD
 *
 * Overview:    Legge i secondi dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_seconds(void) {
    unsigned char _seconds;
    _seconds = EERandomRead(WRITE_ADDR, SECONDS_ADDR);                                  // leggo il dato dal RTCC
    _seconds = _seconds & 0b01111111;                                                   // maschero i bit utili
    return(_seconds);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_minutes
 * Output:      torna il valore dei minuti codificato in packed BCD
 *
 * Overview:    Legge i minuti dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_minutes(void) {
    unsigned char _minutes;
    _minutes = EERandomRead(WRITE_ADDR, MINUTES_ADDR);                                  // leggo il dato dal RTCC
    _minutes =  _minutes & 0b01111111;                                                  // maschero i bit utili
    return( _minutes);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_hours
 * Output:      torna il valore dele ore codificato in packed BCD
 *
 * Overview:    Legge le ore dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_hours(void) {
    unsigned char _hours;
    _hours = EERandomRead(WRITE_ADDR, HOURS_ADDR);                                      // leggo il dato dal RTCC
    _hours =  _hours & 0b00111111;                                                      // maschero i bit utili
    return( _hours);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_day
 * Output:      torna il valore del giorno codificato in packed BCD
 *
 * Overview:    Legge il giorno dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_day(void) {
    unsigned char _day;
    _day = EERandomRead(WRITE_ADDR, DAY_ADDR);                                          // leggo il dato dal RTCC
    _day = _day & 0b00111111;                                                           // maschero i bit utili
    return(_day);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_month
 * Output:      torna il valore del mese codificato in packed BCD
 *
 * Overview:    Legge il mese dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_month(void) {
    unsigned char _month;
    _month = EERandomRead(WRITE_ADDR, MONTH_ADDR);                                      // leggo il dato dal RTCC
    _month = _month & 0b00011111;                                                       // maschero i bit utili
    return(_month);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_year
 * Output:      torna il valore dell'anno codificato in packed BCD  (da 00 a 99)
 *
 * Overview:    Legge l'anno dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_year(void) {
    unsigned char _year;
    _year = EERandomRead(WRITE_ADDR, YEAR_ADDR);                                        // leggo il dato dal RTCC
    return(_year);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_day_w
 * Output:      torna il valore del giorno della settimana
 *
 * Overview:    Legge il nome del giorno dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_day_w(void) {
    unsigned char _day_w;
    _day_w = EERandomRead(WRITE_ADDR, DAY_WEEK_ADDR);                                   // leggo il dato dal RTCC
    _day_w = _day_w & 0b00000111;                                                       // maschero i bit utili
    return(_day_w);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_minutes_al
 * Output:      torna il valore dei minuti dell'allarme codificato in packed BCD
 *
 * Overview:    Legge i minuti dell'allarme dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_minutes_al(void) {
    unsigned char _minutes_al;
    _minutes_al = EERandomRead(WRITE_ADDR, MINUTES_ALARM_ADDR );                        // leggo il dato dal RTCC
    _minutes_al = _minutes_al & 0b01111111;                                             // maschero i bit utili
    return(_minutes_al);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_hours_al
 * Output:      torna il valore delle ore dell'allarme codificato in packed BCD
 *
 * Overview:    Legge le ore dell'allarme dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_hours_al(void) {
    unsigned char _hours_al;
    _hours_al = EERandomRead(WRITE_ADDR, HOURS_ALARM_ADDR );                            // leggo il dato dal RTCC
    _hours_al = _hours_al & 0b00111111;                                                 // maschero i bit utili
    return(_hours_al);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_day_al
 * Output:      torna il valore del giorno dell'allarme codificato in packed BCD
 *
 * Overview:    Legge il giorno dell'allarme dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_day_al(void) {
    unsigned char _day_al;
    _day_al = EERandomRead(WRITE_ADDR, DAYS_ALARM_ADDR );                               // leggo il dato dal RTCC
    _day_al = _day_al & 0b00111111;                                                     // maschero i bit utili
    return(_day_al);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_read_day_w_al
 * Output:      torna il valore del giorno della settimana dell'allarme codificato in packed BCD
 *
 * Overview:    Legge il giorno della settimana dell'allarme dal RTCC
/* ============================================================================================================================================ */
char RTCC_read_day_w_al(void) {
    unsigned char _day_w_al;
    _day_w_al = EERandomRead(WRITE_ADDR, DAY_WEEK_ALARM_ADDR );                         // leggo il dato dal RTCC
    _day_w_al = _day_w_al & 0b00000111;                                                 // maschero i bit utili
    return(_day_w_al);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_seconds
 * Input:       _seconds: valore dei secondi da scrivere nell' RTCC codificato in packed BCD
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore dei secondi nell' RTCC
/* ============================================================================================================================================ */
char RTCC_write_seconds(char _seconds) {
    return(EEByteWrite(WRITE_ADDR, SECONDS_ADDR, _seconds));                            // scrivo il dato nell'RTCC
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_minutes
 * Input:       _minutes : valore dei minuti da scrivere nell' RTCC codificato in packed BCD
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore dei minuti nell' RTCC
/* ============================================================================================================================================ */
char RTCC_write_minutes(char _minutes) {
    return(EEByteWrite(WRITE_ADDR, MINUTES_ADDR, _minutes));                            // scrivo il dato nell'RTCC
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_hours
 * Input:       _hours : valore delle ore da scrivere nell' RTCC codificato in packed BCD
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore delle ore nell' RTCC
/* ============================================================================================================================================ */
char RTCC_write_hours(char _hours) {
    return(EEByteWrite(WRITE_ADDR, HOURS_ADDR, _hours));                                // scrivo il dato nell'RTCC
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_day
 * Input:       _day : valore del giorno da scrivere nell' RTCC codificato in packed BCD
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore del giorno nell' RTCC
/* ============================================================================================================================================ */
char RTCC_write_day(char _day) {
    return(EEByteWrite(WRITE_ADDR, DAY_ADDR, _day));                                    // scrivo il dato nell'RTCC
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_month
 * Input:       _day : valore del mese da scrivere nell' RTCC codificato in packed BCD
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore del mese nell' RTCC
/* ============================================================================================================================================ */
char RTCC_write_month(char _month) {
    return(EEByteWrite(WRITE_ADDR, MONTH_ADDR, _month));                                // scrivo il dato nell'RTCC
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_year
 * Input:       _year : valore dell'anno da scrivere nell' RTCC codificato in packed BCD
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore dell'anno nell' RTCC
/* ============================================================================================================================================ */

char RTCC_write_year(char _year) {
    return(EEByteWrite(WRITE_ADDR, YEAR_ADDR, _year));                                  // scrivo il dato nell'RTCC
}

/* ============================================================================================================================================= /
 * FUNCTION:     RTCC_write_day_w
 * Input:       _day_w : valore del giorno della settimana da scrivere nell' RTCC codificato in packed BCD
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore del giorno della settimana nell' RTCC
/* ============================================================================================================================================ */

char RTCC_write_day_w(char _day_w) {
    return(EEByteWrite(WRITE_ADDR, DAY_WEEK_ADDR, _day_w));                             // scrivo il dato nell'RTCC
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_minutes_al
 * Input:       _minutes_al : valore dei minuti dell'allarme da scrivere nell' RTCC codificato in packed BCD
 *              _enable: abilitazione o disabilitazione dell'allarme (AL_ON, AL_OFF)
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore dei minuti dell'allarme nell' RTCC
/* ============================================================================================================================================ */

char RTCC_write_minutes_al(char _minutes_al, char _enable) {
    return(EEByteWrite(WRITE_ADDR, MINUTES_ALARM_ADDR, (_minutes_al + _enable)));       // scrivo il dato nell'RTCC + l'abilitazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_hours_al
 * Input:       _hours_al : valore delle ore dell'allarme da scrivere nell' RTCC codificato in packed BCD
 *              _enable: abilitazione o disabilitazione dell'allarme (AL_ON, AL_OFF)
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore delle ore dell'allarme nell' RTCC
/* ============================================================================================================================================ */

char RTCC_write_hours_al(char _hours_al, char _enable) {
    return(EEByteWrite(WRITE_ADDR, HOURS_ALARM_ADDR, (_hours_al + _enable)));           // scrivo il dato nell'RTCC + l'abilitazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_day_al
 * Input:       _day_al : valore del giorno dell'allarme da scrivere nell' RTCC codificato in packed BCD
 *              _enable: abilitazione o disabilitazione dell'allarme (AL_ON, AL_OFF)
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore del giorno dell'allarme nell' RTCC
/* ============================================================================================================================================ */

char RTCC_write_day_al(char _day_al, char _enable) {
    return(EEByteWrite(WRITE_ADDR, DAYS_ALARM_ADDR, (_day_al + _enable)));              // scrivo il dato nell'RTCC + l'abilitazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_day_w_al
 * Input:       _day_w_al : valore del giorno della settimana dell'allarme da scrivere nell' RTCC codificato in packed BCD
 *              _enable: abilitazione o disabilitazione dell'allarme (AL_ON, AL_OFF)
 * Output       return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Scrive il valore del giorno della settimana dell'allarme nell' RTCC
/* ============================================================================================================================================ */

char RTCC_write_day_w_al(char _day_w_al, char _enable) {
    return(EEByteWrite(WRITE_ADDR, DAY_WEEK_ALARM_ADDR, (_day_w_al + _enable)));        // scrivo il dato nell'RTCC + l'abilitazione
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_set_alarm_interrupt
 * Input:       _flag : Attiva o disattiva l'interrupt di allarme (ON (1), OFF(2))
 * Output:      return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Attiva o disattiva l'interrupt di allarme avvenuto a seconda dell'ingresso
/* ============================================================================================================================================ */

char RTCC_set_alarm_interrupt(char _flag) {
    char _return;
    if (_flag) {                                                                        // se _flag = 1
        _return = (EEByteWrite(WRITE_ADDR, CONTROL_REG_2_ADDR, 0x02));                  // attivo l'interrupt
    }
        if (!_flag) {                                                                   // se _flag = 0
        _return = (EEByteWrite(WRITE_ADDR, CONTROL_REG_2_ADDR, 0x00));                  // disattivo interrupt
    }
    return(_return);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_clear_alarm
 * Output:      return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Resetta flag di allarme avvenuto nel RTCC
/* ============================================================================================================================================ */

char RTCC_clear_alarm(void) {
    char _return;
    char _state = EERandomRead(WRITE_ADDR, CONTROL_REG_2_ADDR);                         // leggo lo stato attuale di control_status_2

    if (_state & 0b00001000) {                                                          // se avvenuto un allarme
        _return = (EEByteWrite(WRITE_ADDR, CONTROL_REG_2_ADDR, (_state & 0b11110111))); // resetto il bit dedicato nel control_status_2 mantnendo gli
    }                                                                                   // altri pit invariati
    return(_return);
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_clock_out
 * Input:       _setting: settaggio del clock in uscita: CLK_OFF, 32.768_KHz, 1.024_KHz, 32_Hz, 1_Hz
 * Output:      return  1: The info has been properly written
 *              return -1: Bus Collision error
 *              return -2: Not Ack error condition
 *              return -3: Write collision
 *
 * Overview:    Imposta o disattiva il clock in uscita.
/* ============================================================================================================================================ */

char RTCC_clock_out (char _setting) {
        return(EEByteWrite(WRITE_ADDR, CLOCKOUT_REG_ADR, _setting));                    // scrivo il dato nell'RTCC
}